@@ -37,7 +37,6 @@ import org.apache.lucene.codecs.CodecUtil;
 import org.apache.lucene.codecs.StoredFieldsWriter;
 import org.apache.lucene.index.FieldInfo;
 import org.apache.lucene.index.IndexFileNames;
-import org.apache.lucene.index.IndexableField;
 import org.apache.lucene.index.SegmentInfo;
 import org.apache.lucene.store.ByteBuffersDataOutput;
 import org.apache.lucene.store.DataOutput;
@@ -278,67 +277,52 @@ public final class Lucene50CompressingStoredFieldsWriter extends StoredFieldsWri
   }
 
   @Override
-  public void writeField(FieldInfo info, IndexableField field) throws IOException {
-
+  public void writeField(FieldInfo info, int value) throws IOException {
     ++numStoredFieldsInDoc;
-
-    int bits = 0;
-    final BytesRef bytes;
-    final String string;
-
-    Number number = field.numericValue();
-    if (number != null) {
-      if (number instanceof Byte || number instanceof Short || number instanceof Integer) {
-        bits = NUMERIC_INT;
-      } else if (number instanceof Long) {
-        bits = NUMERIC_LONG;
-      } else if (number instanceof Float) {
-        bits = NUMERIC_FLOAT;
-      } else if (number instanceof Double) {
-        bits = NUMERIC_DOUBLE;
-      } else {
-        throw new IllegalArgumentException("cannot store numeric type " + number.getClass());
-      }
-      string = null;
-      bytes = null;
-    } else {
-      bytes = field.binaryValue();
-      if (bytes != null) {
-        bits = BYTE_ARR;
-        string = null;
-      } else {
-        bits = STRING;
-        string = field.stringValue();
-        if (string == null) {
-          throw new IllegalArgumentException(
-              "field "
-                  + field.name()
-                  + " is stored but does not have binaryValue, stringValue nor numericValue");
-        }
-      }
-    }
-
-    final long infoAndBits = (((long) info.number) << TYPE_BITS) | bits;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | NUMERIC_INT;
     bufferedDocs.writeVLong(infoAndBits);
+    bufferedDocs.writeZInt(value);
+  }
 
-    if (bytes != null) {
-      bufferedDocs.writeVInt(bytes.length);
-      bufferedDocs.writeBytes(bytes.bytes, bytes.offset, bytes.length);
-    } else if (string != null) {
-      bufferedDocs.writeString(string);
-    } else {
-      if (number instanceof Byte || number instanceof Short || number instanceof Integer) {
-        bufferedDocs.writeZInt(number.intValue());
-      } else if (number instanceof Long) {
-        writeTLong(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), number.longValue());
-      } else if (number instanceof Float) {
-        writeZFloat(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), number.floatValue());
-      } else if (number instanceof Double) {
-        writeZDouble(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), number.doubleValue());
-      } else {
-        throw new AssertionError("Cannot get here");
-      }
-    }
+  @Override
+  public void writeField(FieldInfo info, long value) throws IOException {
+    ++numStoredFieldsInDoc;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | NUMERIC_LONG;
+    bufferedDocs.writeVLong(infoAndBits);
+    writeTLong(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), value);
+  }
+
+  @Override
+  public void writeField(FieldInfo info, float value) throws IOException {
+    ++numStoredFieldsInDoc;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | NUMERIC_FLOAT;
+    bufferedDocs.writeVLong(infoAndBits);
+    writeZFloat(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), value);
+  }
+
+  @Override
+  public void writeField(FieldInfo info, double value) throws IOException {
+    ++numStoredFieldsInDoc;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | NUMERIC_DOUBLE;
+    bufferedDocs.writeVLong(infoAndBits);
+    writeZDouble(EndiannessReverserUtil.wrapDataOutput(bufferedDocs), value);
+  }
+
+  @Override
+  public void writeField(FieldInfo info, BytesRef value) throws IOException {
+    ++numStoredFieldsInDoc;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | BYTE_ARR;
+    bufferedDocs.writeVLong(infoAndBits);
+    bufferedDocs.writeVInt(value.length);
+    bufferedDocs.writeBytes(value.bytes, value.offset, value.length);
+  }
+
+  @Override
+  public void writeField(FieldInfo info, String value) throws IOException {
+    ++numStoredFieldsInDoc;
+    final long infoAndBits = (((long) info.number) << TYPE_BITS) | STRING;
+    bufferedDocs.writeVLong(infoAndBits);
+    bufferedDocs.writeString(value);
   }
 
   // -0 isn't compressed.
