@@ -29,6 +29,7 @@ import java.util.List;
 import org.apache.lucene.codecs.CodecUtil;
 import org.apache.lucene.codecs.KnnFieldVectorsWriter;
 import org.apache.lucene.codecs.KnnVectorsWriter;
+import org.apache.lucene.codecs.hnsw.DefaultFlatVectorScorer;
 import org.apache.lucene.codecs.lucene90.IndexedDISI;
 import org.apache.lucene.index.ByteVectorValues;
 import org.apache.lucene.index.DocsWithFieldSet;
@@ -429,6 +430,7 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       // TODO: separate random access vector values from DocIdSetIterator?
       int byteSize = fieldInfo.getVectorDimension() * fieldInfo.getVectorEncoding().byteSize;
       OnHeapHnswGraph graph = null;
+      DefaultFlatVectorScorer defaultFlatVectorScorer = new DefaultFlatVectorScorer();
       if (docsWithField.cardinality() != 0) {
         // build graph
         switch (fieldInfo.getVectorEncoding()) {
@@ -438,10 +440,11 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
                     fieldInfo.getVectorDimension(),
                     docsWithField.cardinality(),
                     vectorDataInput,
+                    fieldInfo.getVectorSimilarityFunction(),
                     byteSize);
             RandomVectorScorerSupplier scorerBytesSupplier =
-                RandomVectorScorerSupplier.createBytes(
-                    bytesValues, fieldInfo.getVectorSimilarityFunction());
+                defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                    fieldInfo.getVectorSimilarityFunction(), bytesValues);
             HnswGraphBuilder bytesGraphBuilder =
                 HnswGraphBuilder.create(
                     scorerBytesSupplier, M, beamWidth, HnswGraphBuilder.randSeed);
@@ -455,10 +458,11 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
                     fieldInfo.getVectorDimension(),
                     docsWithField.cardinality(),
                     vectorDataInput,
+                    fieldInfo.getVectorSimilarityFunction(),
                     byteSize);
             RandomVectorScorerSupplier scorerSupplier =
-                RandomVectorScorerSupplier.createFloats(
-                    vectorValues, fieldInfo.getVectorSimilarityFunction());
+                defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                    fieldInfo.getVectorSimilarityFunction(), vectorValues);
             HnswGraphBuilder hnswGraphBuilder =
                 HnswGraphBuilder.create(scorerSupplier, M, beamWidth, HnswGraphBuilder.randSeed);
             graph = hnswGraphBuilder.build(vectorValues.size());
@@ -684,21 +688,21 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       this.dim = fieldInfo.getVectorDimension();
       this.docsWithField = new DocsWithFieldSet();
       vectors = new ArrayList<>();
-      RandomAccessVectorValues<T> raVectors = new RAVectorValues<>(vectors, dim);
+      DefaultFlatVectorScorer defaultFlatVectorScorer = new DefaultFlatVectorScorer();
       final RandomVectorScorerSupplier scorerSupplier;
       switch (fieldInfo.getVectorEncoding()) {
         case BYTE:
           scorerSupplier =
-              RandomVectorScorerSupplier.createBytes(
-                  (RandomAccessVectorValues<byte[]>) raVectors,
-                  fieldInfo.getVectorSimilarityFunction());
+              defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                  fieldInfo.getVectorSimilarityFunction(),
+                  RandomAccessVectorValues.fromBytes((List<byte[]>) vectors, dim));
           break;
 
         case FLOAT32:
           scorerSupplier =
-              RandomVectorScorerSupplier.createFloats(
-                  (RandomAccessVectorValues<float[]>) raVectors,
-                  fieldInfo.getVectorSimilarityFunction());
+              defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                  fieldInfo.getVectorSimilarityFunction(),
+                  RandomAccessVectorValues.fromFloats((List<float[]>) vectors, dim));
           break;
 
         default:
@@ -728,9 +732,9 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       lastDocID = docID;
     }
 
-    OnHeapHnswGraph getGraph() {
+    OnHeapHnswGraph getGraph() throws IOException {
       if (vectors.size() > 0) {
-        return hnswGraphBuilder.getGraph();
+        return hnswGraphBuilder.getCompletedGraph();
       } else {
         return null;
       }
@@ -747,34 +751,4 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
           + hnswGraphBuilder.getGraph().ramBytesUsed();
     }
   }
-
-  private static class RAVectorValues<T> implements RandomAccessVectorValues<T> {
-    private final List<T> vectors;
-    private final int dim;
-
-    RAVectorValues(List<T> vectors, int dim) {
-      this.vectors = vectors;
-      this.dim = dim;
-    }
-
-    @Override
-    public int size() {
-      return vectors.size();
-    }
-
-    @Override
-    public int dimension() {
-      return dim;
-    }
-
-    @Override
-    public T vectorValue(int targetOrd) throws IOException {
-      return vectors.get(targetOrd);
-    }
-
-    @Override
-    public RAVectorValues<T> copy() throws IOException {
-      return this;
-    }
-  }
 }
