@@ -21,21 +21,22 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 import org.apache.lucene.codecs.CodecUtil;
 import org.apache.lucene.codecs.FieldsProducer;
 import org.apache.lucene.codecs.PostingsReaderBase;
 import org.apache.lucene.index.CorruptIndexException;
 import org.apache.lucene.index.FieldInfo;
+import org.apache.lucene.index.FieldInfos;
 import org.apache.lucene.index.IndexFileNames;
 import org.apache.lucene.index.IndexOptions;
 import org.apache.lucene.index.SegmentReadState;
 import org.apache.lucene.index.Terms;
+import org.apache.lucene.internal.hppc.IntCursor;
+import org.apache.lucene.internal.hppc.IntObjectHashMap;
 import org.apache.lucene.store.ChecksumIndexInput;
 import org.apache.lucene.store.IOContext;
 import org.apache.lucene.store.IndexInput;
 import org.apache.lucene.util.BytesRef;
-import org.apache.lucene.util.CollectionUtil;
 import org.apache.lucene.util.IOUtils;
 import org.apache.lucene.util.fst.ByteSequenceOutputs;
 import org.apache.lucene.util.fst.Outputs;
@@ -113,7 +114,8 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
   // produce DocsEnum on demand
   final PostingsReaderBase postingsReader;
 
-  private final Map<String, FieldReader> fieldMap;
+  private final FieldInfos fieldInfos;
+  private final IntObjectHashMap<FieldReader> fieldMap;
   private final List<String> fieldList;
 
   final String segment;
@@ -155,7 +157,7 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
       // Read per-field details
       String metaName =
           IndexFileNames.segmentFileName(segment, state.segmentSuffix, TERMS_META_EXTENSION);
-      Map<String, FieldReader> fieldMap = null;
+      IntObjectHashMap<FieldReader> fieldMap = null;
       Throwable priorE = null;
       long indexLength = -1, termsLength = -1;
       try (ChecksumIndexInput metaIn =
@@ -174,7 +176,7 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
           if (numFields < 0) {
             throw new CorruptIndexException("invalid numFields: " + numFields, metaIn);
           }
-          fieldMap = CollectionUtil.newHashMap(numFields);
+          fieldMap = new IntObjectHashMap<>(numFields);
           for (int i = 0; i < numFields; ++i) {
             final int field = metaIn.readVInt();
             final long numTerms = metaIn.readVLong();
@@ -197,6 +199,11 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
             final int docCount = metaIn.readVInt();
             BytesRef minTerm = readBytesRef(metaIn);
             BytesRef maxTerm = readBytesRef(metaIn);
+            if (numTerms == 1) {
+              assert maxTerm.equals(minTerm);
+              // save heap for edge case of a single term only so min == max
+              maxTerm = minTerm;
+            }
             if (docCount < 0
                 || docCount > state.segmentInfo.maxDoc()) { // #docs with field must be <= #docs
               throw new CorruptIndexException(
@@ -215,7 +222,7 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
             final long indexStartFP = metaIn.readVLong();
             FieldReader previous =
                 fieldMap.put(
-                    fieldInfo.name,
+                    fieldInfo.number,
                     new FieldReader(
                         this,
                         fieldInfo,
@@ -249,10 +256,9 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
       // correct
       CodecUtil.retrieveChecksum(indexIn, indexLength);
       CodecUtil.retrieveChecksum(termsIn, termsLength);
-      List<String> fieldList = new ArrayList<>(fieldMap.keySet());
-      fieldList.sort(null);
+      fieldInfos = state.fieldInfos;
       this.fieldMap = fieldMap;
-      this.fieldList = Collections.unmodifiableList(fieldList);
+      this.fieldList = sortFieldNames(fieldMap, state.fieldInfos);
       success = true;
     } finally {
       if (!success) {
@@ -268,14 +274,23 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
       throw new CorruptIndexException("invalid bytes length: " + numBytes, in);
     }
 
-    BytesRef bytes = new BytesRef();
+    BytesRef bytes = new BytesRef(numBytes);
     bytes.length = numBytes;
-    bytes.bytes = new byte[numBytes];
     in.readBytes(bytes.bytes, 0, numBytes);
 
     return bytes;
   }
 
+  private static List<String> sortFieldNames(
+      IntObjectHashMap<FieldReader> fieldMap, FieldInfos fieldInfos) {
+    List<String> fieldNames = new ArrayList<>(fieldMap.size());
+    for (IntCursor fieldNumber : fieldMap.keys()) {
+      fieldNames.add(fieldInfos.fieldInfo(fieldNumber.value).name);
+    }
+    fieldNames.sort(null);
+    return Collections.unmodifiableList(fieldNames);
+  }
+
   // for debugging
   // private static String toHex(int v) {
   //   return "0x" + Integer.toHexString(v);
@@ -300,7 +315,8 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
   @Override
   public Terms terms(String field) throws IOException {
     assert field != null;
-    return fieldMap.get(field);
+    FieldInfo fieldInfo = fieldInfos.fieldInfo(field);
+    return fieldInfo == null ? null : fieldMap.get(fieldInfo.number);
   }
 
   @Override
@@ -308,24 +324,6 @@ public final class Lucene90BlockTreeTermsReader extends FieldsProducer {
     return fieldMap.size();
   }
 
-  // for debugging
-  String brToString(BytesRef b) {
-    if (b == null) {
-      return "null";
-    } else {
-      try {
-        return b.utf8ToString() + " " + b;
-      } catch (
-          @SuppressWarnings("unused")
-          Throwable t) {
-        // If BytesRef isn't actually UTF8, or it's eg a
-        // prefix of UTF8 that ends mid-unicode-char, we
-        // fallback to hex:
-        return b.toString();
-      }
-    }
-  }
-
   @Override
   public void checkIntegrity() throws IOException {
     // terms index
