@@ -36,6 +36,7 @@ import org.apache.lucene.index.MultiDocValues.MultiSortedDocValues;
 import org.apache.lucene.index.MultiDocValues.MultiSortedSetDocValues;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.KnnCollector;
+import org.apache.lucene.search.VectorScorer;
 import org.apache.lucene.util.ArrayUtil;
 import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.IOUtils;
@@ -892,6 +893,11 @@ final class SlowCompositeCodecReaderWrapper extends CodecReader {
         public int advance(int target) throws IOException {
           return mergedIterator.advance(target);
         }
+
+        @Override
+        public VectorScorer scorer(float[] target) {
+          throw new UnsupportedOperationException();
+        }
       };
     }
 
@@ -946,6 +952,11 @@ final class SlowCompositeCodecReaderWrapper extends CodecReader {
         public int advance(int target) throws IOException {
           return mergedIterator.advance(target);
         }
+
+        @Override
+        public VectorScorer scorer(byte[] target) {
+          throw new UnsupportedOperationException();
+        }
       };
     }
 
