@@ -19,9 +19,11 @@ package org.apache.lucene.search;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.EnumMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.similarities.Similarity;
@@ -157,6 +159,7 @@ final class BooleanWeight extends Weight {
   }
 
   static BulkScorer disableScoring(final BulkScorer scorer) {
+    Objects.requireNonNull(scorer);
     return new BulkScorer() {
 
       @Override
@@ -192,7 +195,7 @@ final class BooleanWeight extends Weight {
   // pkg-private for forcing use of BooleanScorer in tests
   BulkScorer optionalBulkScorer(LeafReaderContext context) throws IOException {
     if (scoreMode == ScoreMode.TOP_SCORES) {
-      if (!query.isPureDisjunction() || weightedClauses.size() > 2) {
+      if (!query.isPureDisjunction()) {
         return null;
       }
 
@@ -218,34 +221,7 @@ final class BooleanWeight extends Weight {
         optionalScorers.add(ss.get(Long.MAX_VALUE));
       }
 
-      return new BulkScorer() {
-        final Scorer bmmScorer = new BlockMaxMaxscoreScorer(BooleanWeight.this, optionalScorers);
-        final DocIdSetIterator iterator = bmmScorer.iterator();
-
-        @Override
-        public int score(LeafCollector collector, Bits acceptDocs, int min, int max)
-            throws IOException {
-          collector.setScorer(bmmScorer);
-
-          int doc = bmmScorer.docID();
-          if (doc < min) {
-            doc = iterator.advance(min);
-          }
-          while (doc < max) {
-            if (acceptDocs == null || acceptDocs.get(doc)) {
-              collector.collect(doc);
-            }
-
-            doc = iterator.nextDoc();
-          }
-          return doc;
-        }
-
-        @Override
-        public long cost() {
-          return iterator.cost();
-        }
-      };
+      return new MaxScoreBulkScorer(context.reader().maxDoc(), optionalScorers);
     }
 
     List<BulkScorer> optional = new ArrayList<BulkScorer>();
@@ -278,31 +254,109 @@ final class BooleanWeight extends Weight {
         this, optional, Math.max(1, query.getMinimumNumberShouldMatch()), scoreMode.needsScores());
   }
 
-  // Return a BulkScorer for the required clauses only,
-  // or null if it is not applicable
+  // Return a BulkScorer for the required clauses only
   private BulkScorer requiredBulkScorer(LeafReaderContext context) throws IOException {
-    BulkScorer scorer = null;
-
+    // Is there a single required clause by any chance? Then pull its bulk scorer.
+    List<WeightedBooleanClause> requiredClauses = new ArrayList<>();
     for (WeightedBooleanClause wc : weightedClauses) {
-      Weight w = wc.weight;
-      BooleanClause c = wc.clause;
-      if (c.isRequired() == false) {
-        continue;
-      }
-      if (scorer != null) {
-        // we don't have a BulkScorer for conjunctions
-        return null;
-      }
-      scorer = w.bulkScorer(context);
-      if (scorer == null) {
-        // no matches
-        return null;
-      }
-      if (c.isScoring() == false && scoreMode.needsScores()) {
-        scorer = disableScoring(scorer);
+      if (wc.clause.isRequired()) {
+        requiredClauses.add(wc);
       }
     }
-    return scorer;
+
+    if (requiredClauses.isEmpty()) {
+      // No required clauses at all.
+      return null;
+    } else if (requiredClauses.size() == 1) {
+      WeightedBooleanClause clause = requiredClauses.get(0);
+      BulkScorer scorer = clause.weight.bulkScorer(context);
+      if (scorer == null) {
+        return null;
+      }
+      if (clause.clause.isScoring() == false && scoreMode.needsScores()) {
+        scorer = disableScoring(scorer);
+      }
+      return scorer;
+    }
+
+    List<ScorerSupplier> requiredNoScoringSupplier = new ArrayList<>();
+    List<ScorerSupplier> requiredScoringSupplier = new ArrayList<>();
+
+    long leadCost = Long.MAX_VALUE;
+    for (WeightedBooleanClause wc : requiredClauses) {
+      Weight w = wc.weight;
+      BooleanClause c = wc.clause;
+      ScorerSupplier scorerSupplier = w.scorerSupplier(context);
+      if (scorerSupplier == null) {
+        // One clause doesn't have matches, so the entire conjunction doesn't have matches.
+        return null;
+      }
+      leadCost = Math.min(leadCost, scorerSupplier.cost());
+
+      if (c.isScoring() && scoreMode.needsScores()) {
+        requiredScoringSupplier.add(scorerSupplier);
+      } else {
+        requiredNoScoringSupplier.add(scorerSupplier);
+      }
+    }
+
+    List<Scorer> requiredNoScoring = new ArrayList<>();
+    for (ScorerSupplier ss : requiredNoScoringSupplier) {
+      requiredNoScoring.add(ss.get(leadCost));
+    }
+    List<Scorer> requiredScoring = new ArrayList<>();
+    for (ScorerSupplier ss : requiredScoringSupplier) {
+      if (requiredScoringSupplier.size() == 1) {
+        ss.setTopLevelScoringClause();
+      }
+      requiredScoring.add(ss.get(leadCost));
+    }
+    if (scoreMode == ScoreMode.TOP_SCORES
+        && requiredNoScoringSupplier.isEmpty()
+        && requiredScoring.size() > 1
+        // Only specialize top-level conjunctions for clauses that don't have a two-phase iterator.
+        && requiredScoring.stream().map(Scorer::twoPhaseIterator).allMatch(Objects::isNull)) {
+      return new BlockMaxConjunctionBulkScorer(context.reader().maxDoc(), requiredScoring);
+    }
+    if (scoreMode != ScoreMode.TOP_SCORES
+        && requiredScoring.size() + requiredNoScoring.size() >= 2
+        && requiredScoring.stream().map(Scorer::twoPhaseIterator).allMatch(Objects::isNull)
+        && requiredNoScoring.stream().map(Scorer::twoPhaseIterator).allMatch(Objects::isNull)) {
+      return new ConjunctionBulkScorer(requiredScoring, requiredNoScoring);
+    }
+    if (scoreMode == ScoreMode.TOP_SCORES && requiredScoring.size() > 1) {
+      requiredScoring =
+          Collections.singletonList(new BlockMaxConjunctionScorer(this, requiredScoring));
+    }
+    Scorer conjunctionScorer;
+    if (requiredNoScoring.size() + requiredScoring.size() == 1) {
+      if (requiredScoring.size() == 1) {
+        conjunctionScorer = requiredScoring.get(0);
+      } else {
+        conjunctionScorer = requiredNoScoring.get(0);
+        if (scoreMode.needsScores()) {
+          Scorer inner = conjunctionScorer;
+          conjunctionScorer =
+              new FilterScorer(inner) {
+                @Override
+                public float score() throws IOException {
+                  return 0f;
+                }
+
+                @Override
+                public float getMaxScore(int upTo) throws IOException {
+                  return 0f;
+                }
+              };
+        }
+      }
+    } else {
+      List<Scorer> required = new ArrayList<>();
+      required.addAll(requiredScoring);
+      required.addAll(requiredNoScoring);
+      conjunctionScorer = new ConjunctionScorer(this, required, requiredScoring);
+    }
+    return new DefaultBulkScorer(conjunctionScorer);
   }
 
   /**
@@ -342,7 +396,7 @@ final class BooleanWeight extends Weight {
         return null;
       }
 
-    } else if (numRequiredClauses == 1
+    } else if (numRequiredClauses > 0
         && numOptionalClauses == 0
         && query.getMinimumNumberShouldMatch() == 0) {
       positiveScorer = requiredBulkScorer(context);
@@ -401,8 +455,9 @@ final class BooleanWeight extends Weight {
     final int numDocs = context.reader().numDocs();
     int positiveCount;
     if (query.isPureDisjunction()) {
-      positiveCount = optCount(context, Occur.SHOULD);
-    } else if ((query.getClauses(Occur.FILTER).isEmpty() == false
+      return optCount(context, Occur.SHOULD);
+    }
+    if ((query.getClauses(Occur.FILTER).isEmpty() == false
             || query.getClauses(Occur.MUST).isEmpty() == false)
         && query.getMinimumNumberShouldMatch() == 0) {
       positiveCount = reqCount(context);
@@ -469,14 +524,18 @@ final class BooleanWeight extends Weight {
   private int optCount(LeafReaderContext context, Occur occur) throws IOException {
     final int numDocs = context.reader().numDocs();
     int optCount = 0;
+    boolean unknownCount = false;
     for (WeightedBooleanClause weightedClause : weightedClauses) {
       if (weightedClause.clause.getOccur() != occur) {
         continue;
       }
       int count = weightedClause.weight.count(context);
-      if (count == -1 || count == numDocs) {
-        // If any of the clauses has a number of matches that is unknown, the number of matches of
-        // the disjunction is unknown.
+      if (count == -1) {
+        // If one clause has a number of matches that is unknown, let's be more aggressive to check
+        // whether remain clauses could match all docs.
+        unknownCount = true;
+        continue;
+      } else if (count == numDocs) {
         // If either clause matches all docs, then the disjunction matches all docs.
         return count;
       } else if (count == 0) {
@@ -488,10 +547,13 @@ final class BooleanWeight extends Weight {
       } else {
         // We have two clauses whose count is in [1, numDocs), we can't figure out the number of
         // docs that match the disjunction without running the query.
-        return -1;
+        unknownCount = true;
       }
     }
-    return optCount;
+    // If at least one of clauses has a number of matches that is unknown and no clause matches all
+    // docs, then the number of matches of
+    // the disjunction is unknown
+    return unknownCount ? -1 : optCount;
   }
 
   @Override
@@ -505,7 +567,8 @@ final class BooleanWeight extends Weight {
 
   @Override
   public boolean isCacheable(LeafReaderContext ctx) {
-    if (query.clauses().size() > TermInSetQuery.BOOLEAN_REWRITE_TERM_COUNT_THRESHOLD) {
+    if (query.clauses().size()
+        > AbstractMultiTermQueryConstantScoreWrapper.BOOLEAN_REWRITE_TERM_COUNT_THRESHOLD) {
       // Disallow caching large boolean queries to not encourage users
       // to build large boolean queries as a workaround to the fact that
       // we disallow caching large TermInSetQueries.
