@@ -30,6 +30,7 @@ import org.apache.lucene.index.Impact;
 import org.apache.lucene.index.Impacts;
 import org.apache.lucene.index.ImpactsSource;
 import org.apache.lucene.index.Term;
+import org.apache.lucene.internal.hppc.IntHashSet;
 import org.apache.lucene.search.similarities.Similarity.SimScorer;
 import org.apache.lucene.util.FixedBitSet;
 
@@ -565,9 +566,9 @@ public final class SloppyPhraseMatcher extends PhraseMatcher {
       ArrayList<FixedBitSet> bb = ppTermsBitSets(rpp, rptTerms);
       unionTermGroups(bb);
       HashMap<Term, Integer> tg = termGroups(rptTerms, bb);
-      HashSet<Integer> distinctGroupIDs = new HashSet<>(tg.values());
-      for (int i = 0; i < distinctGroupIDs.size(); i++) {
-        tmp.add(new HashSet<PhrasePositions>());
+      int numDistinctGroupIds = new IntHashSet(tg.values()).size();
+      for (int i = 0; i < numDistinctGroupIds; i++) {
+        tmp.add(new HashSet<>());
       }
       for (PhrasePositions pp : rpp) {
         for (Term t : pp.terms) {
