@@ -116,11 +116,17 @@ public final class SynonymQuery extends Query {
     this.field = Objects.requireNonNull(field);
   }
 
+  /** Returns the terms of this {@link SynonymQuery} */
   public List<Term> getTerms() {
     return Collections.unmodifiableList(
         Arrays.stream(terms).map(t -> new Term(field, t.term)).collect(Collectors.toList()));
   }
 
+  /** Returns the field name of this {@link SynonymQuery} */
+  public String getField() {
+    return field;
+  }
+
   @Override
   public String toString(String field) {
     StringBuilder builder = new StringBuilder("Synonym(");
