@@ -354,6 +354,30 @@ public final class ArrayUtil {
     return growExact(array, Math.min(maxLength, potentialLength));
   }
 
+  /**
+   * Returns an array whose size is at least {@code minLength} but not over {@code maxLength},
+   * growing exponentially if it needs to grow.
+   */
+  public static float[] growInRange(float[] array, int minLength, int maxLength) {
+    assert minLength >= 0
+        : "minLength must be positive (got " + minLength + "): likely integer overflow?";
+
+    if (minLength > maxLength) {
+      throw new IllegalArgumentException(
+          "requested minimum array length "
+              + minLength
+              + " is larger than requested maximum array length "
+              + maxLength);
+    }
+
+    if (array.length >= minLength) {
+      return array;
+    }
+
+    int potentialLength = oversize(minLength, Float.BYTES);
+    return growExact(array, Math.min(maxLength, potentialLength));
+  }
+
   /**
    * Returns an array whose size is at least {@code minSize}, generally over-allocating
    * exponentially
@@ -623,6 +647,11 @@ public final class ArrayUtil {
     }.select(from, to, k);
   }
 
+  /** Copies an array into a new array. */
+  public static byte[] copyArray(byte[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -636,6 +665,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static char[] copyArray(char[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -649,6 +683,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static short[] copyArray(short[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -662,6 +701,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static int[] copyArray(int[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -675,6 +719,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static long[] copyArray(long[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -688,6 +737,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static float[] copyArray(float[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -701,6 +755,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static double[] copyArray(double[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
@@ -714,6 +773,11 @@ public final class ArrayUtil {
     return copy;
   }
 
+  /** Copies an array into a new array. */
+  public static <T> T[] copyArray(T[] array) {
+    return copyOfSubArray(array, 0, array.length);
+  }
+
   /**
    * Copies the specified range of the given array into a new sub array.
    *
