@@ -19,9 +19,10 @@ package org.apache.lucene.util.hnsw;
 
 import java.io.IOException;
 import java.util.Arrays;
-import java.util.concurrent.locks.ReadWriteLock;
-import java.util.concurrent.locks.ReentrantReadWriteLock;
-import org.apache.lucene.util.ArrayUtil;
+import java.util.function.LongConsumer;
+import org.apache.lucene.internal.hppc.MaxSizedFloatArrayList;
+import org.apache.lucene.internal.hppc.MaxSizedIntArrayList;
+import org.apache.lucene.util.RamUsageEstimator;
 
 /**
  * NeighborArray encodes the neighbors of a node and their mutual scores in the HNSW graph as a pair
@@ -32,17 +33,32 @@ import org.apache.lucene.util.ArrayUtil;
  * @lucene.internal
  */
 public class NeighborArray {
+  private static final long BASE_RAM_BYTES_USED =
+      RamUsageEstimator.shallowSizeOfInstance(NeighborArray.class);
+
   private final boolean scoresDescOrder;
   private int size;
-  private final float[] scores;
-  private final int[] nodes;
+  private final int maxSize;
+  private final MaxSizedFloatArrayList scores;
+  private final MaxSizedIntArrayList nodes;
   private int sortedNodeSize;
-  public final ReadWriteLock rwlock = new ReentrantReadWriteLock(true);
+  private long ramBytesUsed = BASE_RAM_BYTES_USED;
+  private final LongConsumer onHeapMemoryUsageListener;
 
   public NeighborArray(int maxSize, boolean descOrder) {
-    nodes = new int[maxSize];
-    scores = new float[maxSize];
+    this(maxSize, descOrder, null);
+  }
+
+  public NeighborArray(int maxSize, boolean descOrder, LongConsumer onHeapMemoryUsageListener) {
+    this.maxSize = maxSize;
+    nodes = new MaxSizedIntArrayList(maxSize, maxSize / 8);
+    scores = new MaxSizedFloatArrayList(maxSize, maxSize / 8);
+    this.ramBytesUsed += nodes.ramBytesUsed() + scores.ramBytesUsed();
     this.scoresDescOrder = descOrder;
+    this.onHeapMemoryUsageListener = onHeapMemoryUsageListener;
+    if (onHeapMemoryUsageListener != null) {
+      onHeapMemoryUsageListener.accept(ramBytesUsed);
+    }
   }
 
   /**
@@ -51,35 +67,46 @@ public class NeighborArray {
    */
   public void addInOrder(int newNode, float newScore) {
     assert size == sortedNodeSize : "cannot call addInOrder after addOutOfOrder";
-    if (size == nodes.length) {
+    if (size == maxSize) {
       throw new IllegalStateException("No growth is allowed");
     }
     if (size > 0) {
-      float previousScore = scores[size - 1];
+      float previousScore = scores.get(size - 1);
       assert ((scoresDescOrder && (previousScore >= newScore))
               || (scoresDescOrder == false && (previousScore <= newScore)))
           : "Nodes are added in the incorrect order! Comparing "
               + newScore
               + " to "
-              + Arrays.toString(ArrayUtil.copyOfSubArray(scores, 0, size));
+              + Arrays.toString(scores.toArray());
     }
-    nodes[size] = newNode;
-    scores[size] = newScore;
+    int previousLength = nodes.buffer.length;
+    nodes.add(newNode);
+    scores.add(newScore);
+    alertOnHeapMemoryUsageChange(nodes.buffer.length, previousLength);
     ++size;
     ++sortedNodeSize;
   }
 
   /** Add node and newScore but do not insert as sorted */
   public void addOutOfOrder(int newNode, float newScore) {
-    if (size == nodes.length) {
+    if (size == maxSize) {
       throw new IllegalStateException("No growth is allowed");
     }
-
-    scores[size] = newScore;
-    nodes[size] = newNode;
+    int previousLength = nodes.buffer.length;
+    nodes.add(newNode);
+    scores.add(newScore);
+    alertOnHeapMemoryUsageChange(nodes.buffer.length, previousLength);
     size++;
   }
 
+  private void alertOnHeapMemoryUsageChange(int newLength, int previousLength) {
+    if (newLength > previousLength && onHeapMemoryUsageListener != null) {
+      int lengthDelta = newLength - previousLength;
+      onHeapMemoryUsageListener.accept(
+          (long) (lengthDelta) * Integer.BYTES + (long) (lengthDelta) * Float.BYTES);
+    }
+  }
+
   /**
    * In addition to {@link #addOutOfOrder(int, float)}, this function will also remove the
    * least-diverse node if the node array is full after insertion
@@ -93,12 +120,12 @@ public class NeighborArray {
       int newNode, float newScore, int nodeId, RandomVectorScorerSupplier scorerSupplier)
       throws IOException {
     addOutOfOrder(newNode, newScore);
-    if (size < nodes.length) {
+    if (size < maxSize) {
       return;
     }
     // we're oversize, need to do diversity check and pop out the least diverse neighbour
     removeIndex(findWorstNonDiverse(nodeId, scorerSupplier));
-    assert size == nodes.length - 1;
+    assert size == maxSize - 1;
   }
 
   /**
@@ -136,8 +163,8 @@ public class NeighborArray {
   /** insert the first unsorted node into its sorted position */
   private int insertSortedInternal(RandomVectorScorer scorer) throws IOException {
     assert sortedNodeSize < size : "Call this method only when there's unsorted node";
-    int tmpNode = nodes[sortedNodeSize];
-    float tmpScore = scores[sortedNodeSize];
+    int tmpNode = nodes.get(sortedNodeSize);
+    float tmpScore = scores.get(sortedNodeSize);
 
     if (Float.isNaN(tmpScore)) {
       tmpScore = scorer.score(tmpNode);
@@ -148,11 +175,19 @@ public class NeighborArray {
             ? descSortFindRightMostInsertionPoint(tmpScore, sortedNodeSize)
             : ascSortFindRightMostInsertionPoint(tmpScore, sortedNodeSize);
     System.arraycopy(
-        nodes, insertionPoint, nodes, insertionPoint + 1, sortedNodeSize - insertionPoint);
+        nodes.buffer,
+        insertionPoint,
+        nodes.buffer,
+        insertionPoint + 1,
+        sortedNodeSize - insertionPoint);
     System.arraycopy(
-        scores, insertionPoint, scores, insertionPoint + 1, sortedNodeSize - insertionPoint);
-    nodes[insertionPoint] = tmpNode;
-    scores[insertionPoint] = tmpScore;
+        scores.buffer,
+        insertionPoint,
+        scores.buffer,
+        insertionPoint + 1,
+        sortedNodeSize - insertionPoint);
+    nodes.buffer[insertionPoint] = tmpNode;
+    scores.buffer[insertionPoint] = tmpScore;
     ++sortedNodeSize;
     return insertionPoint;
   }
@@ -173,19 +208,29 @@ public class NeighborArray {
    * @lucene.internal
    */
   public int[] nodes() {
-    return nodes;
+    return nodes.buffer;
   }
 
-  public float[] scores() {
-    return scores;
+  /**
+   * Get the score at the given index
+   *
+   * @param i index of the score to get
+   * @return the score at the given index
+   */
+  public float getScores(int i) {
+    return scores.get(i);
   }
 
   public void clear() {
     size = 0;
     sortedNodeSize = 0;
+    nodes.clear();
+    scores.clear();
   }
 
   void removeLast() {
+    nodes.removeLast();
+    scores.removeLast();
     size--;
     sortedNodeSize = Math.min(sortedNodeSize, size);
   }
@@ -195,8 +240,8 @@ public class NeighborArray {
       removeLast();
       return;
     }
-    System.arraycopy(nodes, idx + 1, nodes, idx, size - idx - 1);
-    System.arraycopy(scores, idx + 1, scores, idx, size - idx - 1);
+    nodes.removeAt(idx);
+    scores.removeAt(idx);
     if (idx < sortedNodeSize) {
       sortedNodeSize--;
     }
@@ -209,11 +254,11 @@ public class NeighborArray {
   }
 
   private int ascSortFindRightMostInsertionPoint(float newScore, int bound) {
-    int insertionPoint = Arrays.binarySearch(scores, 0, bound, newScore);
+    int insertionPoint = Arrays.binarySearch(scores.buffer, 0, bound, newScore);
     if (insertionPoint >= 0) {
       // find the right most position with the same score
       while ((insertionPoint < bound - 1)
-          && (scores[insertionPoint + 1] == scores[insertionPoint])) {
+          && (scores.get(insertionPoint + 1) == scores.get(insertionPoint))) {
         insertionPoint++;
       }
       insertionPoint++;
@@ -228,7 +273,7 @@ public class NeighborArray {
     int end = bound - 1;
     while (start <= end) {
       int mid = (start + end) / 2;
-      if (scores[mid] < newScore) end = mid - 1;
+      if (scores.get(mid) < newScore) end = mid - 1;
       else start = mid + 1;
     }
     return start;
@@ -265,12 +310,12 @@ public class NeighborArray {
       int uncheckedCursor,
       RandomVectorScorerSupplier scorerSupplier)
       throws IOException {
-    float minAcceptedSimilarity = scores[candidateIndex];
-    RandomVectorScorer scorer = scorerSupplier.scorer(nodes[candidateIndex]);
+    float minAcceptedSimilarity = scores.get(candidateIndex);
+    RandomVectorScorer scorer = scorerSupplier.scorer(nodes.get(candidateIndex));
     if (candidateIndex == uncheckedIndexes[uncheckedCursor]) {
       // the candidate itself is unchecked
       for (int i = candidateIndex - 1; i >= 0; i--) {
-        float neighborSimilarity = scorer.score(nodes[i]);
+        float neighborSimilarity = scorer.score(nodes.get(i));
         // candidate node is too similar to node i given its score relative to the base node
         if (neighborSimilarity >= minAcceptedSimilarity) {
           return true;
@@ -281,7 +326,7 @@ public class NeighborArray {
       // inserted) unchecked nodes
       assert candidateIndex > uncheckedIndexes[uncheckedCursor];
       for (int i = uncheckedCursor; i >= 0; i--) {
-        float neighborSimilarity = scorer.score(nodes[uncheckedIndexes[i]]);
+        float neighborSimilarity = scorer.score(nodes.get(uncheckedIndexes[i]));
         // candidate node is too similar to node i given its score relative to the base node
         if (neighborSimilarity >= minAcceptedSimilarity) {
           return true;
@@ -290,4 +335,8 @@ public class NeighborArray {
     }
     return false;
   }
+
+  public int maxSize() {
+    return maxSize;
+  }
 }
