@@ -21,6 +21,7 @@ import java.io.IOException;
 import java.nio.file.NoSuchFileException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
 import org.apache.lucene.codecs.Codec;
@@ -1547,8 +1548,9 @@ public class TestAddIndexes extends LuceneTestCase {
     IndexReader r3 = w.getReader();
     w.close();
     assertEquals(2, r3.numDocs());
+    StoredFields storedFields = r3.storedFields();
     for (int docID = 0; docID < 2; docID++) {
-      Document d = r3.document(docID);
+      Document d = storedFields.document(docID);
       if (d.get("id").equals("1")) {
         assertEquals("doc1 field1", d.get("f1"));
       } else {
@@ -1814,4 +1816,218 @@ public class TestAddIndexes extends LuceneTestCase {
     assertEquals(wrappedReader.numDocs(), writer.getDocStats().maxDoc);
     IOUtils.close(reader, writer, dir3, dir2, dir1);
   }
+
+  public void testAddIndicesWithBlocks() throws IOException {
+    boolean[] addHasBlocksPerm = {true, true, false, false};
+    boolean[] baseHasBlocksPerm = {true, false, true, false};
+    for (int perm = 0; perm < addHasBlocksPerm.length; perm++) {
+      boolean addHasBlocks = addHasBlocksPerm[perm];
+      boolean baseHasBlocks = baseHasBlocksPerm[perm];
+      try (Directory dir = newDirectory()) {
+        try (RandomIndexWriter writer = new RandomIndexWriter(random(), dir)) {
+          int numBlocks = 1 + random().nextInt(10);
+          for (int i = 0; i < numBlocks; i++) {
+            int numDocs = baseHasBlocks ? 2 + random().nextInt(10) : 1;
+            List<Document> docs = new ArrayList<>();
+            for (int j = 0; j < numDocs; j++) {
+              Document doc = new Document();
+              int value = random().nextInt(5);
+              doc.add(new StringField("value", "" + value, Field.Store.YES));
+              docs.add(doc);
+            }
+            writer.addDocuments(docs);
+          }
+          writer.commit();
+        }
+
+        try (Directory addDir = newDirectory()) {
+          int numBlocks = 1 + random().nextInt(10);
+          try (RandomIndexWriter writer = new RandomIndexWriter(random(), addDir)) {
+            for (int i = 0; i < numBlocks; i++) {
+              int numDocs = addHasBlocks ? 2 + random().nextInt(10) : 1;
+              List<Document> docs = new ArrayList<>();
+              for (int j = 0; j < numDocs; j++) {
+                Document doc = new Document();
+                int value = random().nextInt(5);
+                doc.add(new StringField("value", "" + value, Field.Store.YES));
+                docs.add(doc);
+              }
+              writer.addDocuments(docs);
+            }
+            writer.commit();
+          }
+
+          try (IndexWriter writer = new IndexWriter(dir, newIndexWriterConfig())) {
+            if (random().nextBoolean()) {
+              writer.addIndexes(addDir);
+            } else {
+              try (DirectoryReader reader = DirectoryReader.open(addDir)) {
+                CodecReader[] readers = new CodecReader[(reader.leaves().size())];
+                for (int i = 0; i < readers.length; i++) {
+                  readers[i] = (CodecReader) reader.leaves().get(i).reader();
+                }
+                writer.addIndexes(readers);
+              }
+            }
+            writer.forceMerge(1, true);
+          }
+
+          try (DirectoryReader reader = DirectoryReader.open(dir)) {
+            SegmentReader codecReader = (SegmentReader) reader.leaves().get(0).reader();
+            assertEquals(1, reader.leaves().size());
+            if (addHasBlocks || baseHasBlocks) {
+              assertTrue(
+                  "addHasBlocks: " + addHasBlocks + " baseHasBlocks: " + baseHasBlocks,
+                  codecReader.getSegmentInfo().info.getHasBlocks());
+            } else {
+              assertFalse(codecReader.getSegmentInfo().info.getHasBlocks());
+            }
+          }
+        }
+      }
+    }
+  }
+
+  public void testSetDiagnostics() throws IOException {
+    MergePolicy myMergePolicy =
+        new FilterMergePolicy(newLogMergePolicy(4)) {
+          @Override
+          public MergeSpecification findMerges(CodecReader... readers) throws IOException {
+            MergeSpecification spec = super.findMerges(readers);
+            if (spec == null) {
+              return null;
+            }
+            MergeSpecification newSpec = new MergeSpecification();
+            for (OneMerge merge : spec.merges) {
+              newSpec.add(
+                  new OneMerge(merge) {
+                    @Override
+                    public void setMergeInfo(SegmentCommitInfo info) {
+                      super.setMergeInfo(info);
+                      info.info.addDiagnostics(
+                          Collections.singletonMap("merge_policy", "my_merge_policy"));
+                    }
+                  });
+            }
+            return newSpec;
+          }
+        };
+    Directory sourceDir = newDirectory();
+    try (IndexWriter w = new IndexWriter(sourceDir, newIndexWriterConfig())) {
+      Document doc = new Document();
+      w.addDocument(doc);
+    }
+    DirectoryReader reader = DirectoryReader.open(sourceDir);
+    CodecReader codecReader = SlowCodecReaderWrapper.wrap(reader.leaves().get(0).reader());
+
+    Directory targetDir = newDirectory();
+    try (IndexWriter w =
+        new IndexWriter(targetDir, newIndexWriterConfig().setMergePolicy(myMergePolicy))) {
+      w.addIndexes(codecReader);
+    }
+
+    SegmentInfos si = SegmentInfos.readLatestCommit(targetDir);
+    assertNotEquals(0, si.size());
+    for (SegmentCommitInfo sci : si) {
+      assertEquals(
+          IndexWriter.SOURCE_ADDINDEXES_READERS, sci.info.getDiagnostics().get(IndexWriter.SOURCE));
+      assertEquals("my_merge_policy", sci.info.getDiagnostics().get("merge_policy"));
+    }
+    reader.close();
+    targetDir.close();
+    sourceDir.close();
+  }
+
+  public void testIllegalParentDocChange() throws Exception {
+    Directory dir1 = newDirectory();
+    IndexWriterConfig iwc1 = newIndexWriterConfig(new MockAnalyzer(random()));
+    iwc1.setParentField("foobar");
+    RandomIndexWriter w1 = new RandomIndexWriter(random(), dir1, iwc1);
+    Document parent = new Document();
+    w1.addDocuments(Arrays.asList(new Document(), new Document(), parent));
+    w1.commit();
+    w1.addDocuments(Arrays.asList(new Document(), new Document(), parent));
+    w1.commit();
+    // so the index sort is in fact burned into the index:
+    w1.forceMerge(1);
+    w1.close();
+
+    Directory dir2 = newDirectory();
+    IndexWriterConfig iwc2 = newIndexWriterConfig(new MockAnalyzer(random()));
+    iwc2.setParentField("foo");
+    RandomIndexWriter w2 = new RandomIndexWriter(random(), dir2, iwc2);
+
+    IndexReader r1 = DirectoryReader.open(dir1);
+    String message =
+        expectThrows(
+                IllegalArgumentException.class,
+                () -> {
+                  w2.addIndexes((SegmentReader) getOnlyLeafReader(r1));
+                })
+            .getMessage();
+    assertEquals(
+        "can't add field [foobar] as parent document field; this IndexWriter is configured with [foo] as parent document field",
+        message);
+
+    message =
+        expectThrows(
+                IllegalArgumentException.class,
+                () -> {
+                  w2.addIndexes(dir1);
+                })
+            .getMessage();
+    assertEquals(
+        "can't add field [foobar] as parent document field; this IndexWriter is configured with [foo] as parent document field",
+        message);
+
+    Directory dir3 = newDirectory();
+    IndexWriterConfig iwc3 = newIndexWriterConfig(new MockAnalyzer(random()));
+    iwc3.setParentField("foobar");
+    RandomIndexWriter w3 = new RandomIndexWriter(random(), dir3, iwc3);
+
+    w3.addIndexes((SegmentReader) getOnlyLeafReader(r1));
+    w3.addIndexes(dir1);
+
+    IOUtils.close(r1, dir1, w2, dir2, w3, dir3);
+  }
+
+  public void testIllegalNonParentField() throws IOException {
+    Directory dir1 = newDirectory();
+    IndexWriterConfig iwc1 = newIndexWriterConfig(new MockAnalyzer(random()));
+    RandomIndexWriter w1 = new RandomIndexWriter(random(), dir1, iwc1);
+    Document parent = new Document();
+    parent.add(new StringField("foo", "XXX", Field.Store.NO));
+    w1.addDocument(parent);
+    w1.close();
+
+    Directory dir2 = newDirectory();
+    IndexWriterConfig iwc2 = newIndexWriterConfig(new MockAnalyzer(random()));
+    iwc2.setParentField("foo");
+    RandomIndexWriter w2 = new RandomIndexWriter(random(), dir2, iwc2);
+
+    IndexReader r1 = DirectoryReader.open(dir1);
+    String message =
+        expectThrows(
+                IllegalArgumentException.class,
+                () -> {
+                  w2.addIndexes((SegmentReader) getOnlyLeafReader(r1));
+                })
+            .getMessage();
+    assertEquals(
+        "can't add [foo] as non parent document field; this IndexWriter is configured with [foo] as parent document field",
+        message);
+
+    message =
+        expectThrows(
+                IllegalArgumentException.class,
+                () -> {
+                  w2.addIndexes(dir1);
+                })
+            .getMessage();
+    assertEquals(
+        "can't add [foo] as non parent document field; this IndexWriter is configured with [foo] as parent document field",
+        message);
+
+    IOUtils.close(r1, dir1, w2, dir2);
+  }
 }
