@@ -758,11 +758,12 @@ public class TestBinaryDocValuesUpdates extends LuceneTestCase {
           BinaryDocValues values = leafReader.getBinaryDocValues("number");
           NumericDocValues sortValues = leafReader.getNumericDocValues("sort");
           Bits liveDocs = leafReader.getLiveDocs();
+          StoredFields storedFields = leafReader.storedFields();
 
           long lastSortValue = Long.MIN_VALUE;
           for (int i = 0; i < leafReader.maxDoc(); i++) {
 
-            Document doc = leafReader.document(i);
+            Document doc = storedFields.document(i);
             OneSortDoc sortDoc = docs.get(Integer.parseInt(doc.get("id")));
 
             assertEquals(i, values.nextDoc());
