@@ -20,17 +20,21 @@ import com.carrotsearch.randomizedtesting.generators.RandomStrings;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.Executor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
+import org.apache.lucene.codecs.Codec;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
+import org.apache.lucene.document.KnnFloatVectorField;
 import org.apache.lucene.document.StringField;
 import org.apache.lucene.document.TextField;
 import org.apache.lucene.index.IndexWriterConfig.OpenMode;
@@ -43,6 +47,10 @@ import org.apache.lucene.tests.store.MockDirectoryWrapper;
 import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.tests.util.TestUtil;
 import org.apache.lucene.util.InfoStream;
+import org.apache.lucene.util.SameThreadExecutorService;
+import org.apache.lucene.util.StringHelper;
+import org.apache.lucene.util.SuppressForbidden;
+import org.apache.lucene.util.Version;
 
 public class TestConcurrentMergeScheduler extends LuceneTestCase {
 
@@ -90,12 +98,22 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
                   || (th instanceof IllegalStateException
                       && th.getMessage().contains("this writer hit an unrecoverable error"));
             }
+
+            @Override
+            // override here to ensure even tiny merges get the parallel executor
+            public Executor getIntraMergeExecutor(MergePolicy.OneMerge merge) {
+              assert intraMergeExecutor != null : "intraMergeExecutor is not initialized";
+              return intraMergeExecutor;
+            }
           });
     }
     IndexWriter writer = new IndexWriter(directory, iwc);
     Document doc = new Document();
     Field idField = newStringField("id", "", Field.Store.YES);
+    KnnFloatVectorField knnField = new KnnFloatVectorField("knn", new float[] {0.0f, 0.0f});
     doc.add(idField);
+    // Add knn float vectors to test parallel merge
+    doc.add(knnField);
 
     outer:
     for (int i = 0; i < 10; i++) {
@@ -105,6 +123,7 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
 
       for (int j = 0; j < 20; j++) {
         idField.setStringValue(Integer.toString(i * 20 + j));
+        knnField.setVectorValue(new float[] {random().nextFloat(), random().nextFloat()});
         writer.addDocument(doc);
       }
 
@@ -154,6 +173,7 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
     IndexWriter writer =
         new IndexWriter(
             directory, newIndexWriterConfig(new MockAnalyzer(random())).setMergePolicy(mp));
+    TestUtil.reduceOpenFiles(writer);
 
     Document doc = new Document();
     Field idField = newStringField("id", "", Field.Store.YES);
@@ -225,23 +245,35 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
     Directory directory = newDirectory();
     Document doc = new Document();
     Field idField = newStringField("id", "", Field.Store.YES);
+    KnnFloatVectorField knnField = new KnnFloatVectorField("knn", new float[] {0.0f, 0.0f});
     doc.add(idField);
+    doc.add(knnField);
+    IndexWriterConfig iwc =
+        newIndexWriterConfig(new MockAnalyzer(random()))
+            // Force excessive merging:
+            .setMaxBufferedDocs(2)
+            .setMergePolicy(newLogMergePolicy(100))
+            .setCommitOnClose(false);
+    if (iwc.getMergeScheduler() instanceof ConcurrentMergeScheduler) {
+      iwc.setMergeScheduler(
+          new ConcurrentMergeScheduler() {
+            @Override
+            // override here to ensure even tiny merges get the parallel executor
+            public Executor getIntraMergeExecutor(MergePolicy.OneMerge merge) {
+              assert intraMergeExecutor != null : "scaledExecutor is not initialized";
+              return intraMergeExecutor;
+            }
+          });
+    }
 
-    IndexWriter writer =
-        new IndexWriter(
-            directory,
-            newIndexWriterConfig(new MockAnalyzer(random()))
-                .
-                // Force excessive merging:
-                setMaxBufferedDocs(2)
-                .setMergePolicy(newLogMergePolicy(100))
-                .setCommitOnClose(false));
+    IndexWriter writer = new IndexWriter(directory, iwc);
 
     int numIters = TEST_NIGHTLY ? 10 : 3;
     for (int iter = 0; iter < numIters; iter++) {
 
       for (int j = 0; j < 201; j++) {
         idField.setStringValue(Integer.toString(iter * 201 + j));
+        knnField.setVectorValue(new float[] {random().nextFloat(), random().nextFloat()});
         writer.addDocument(doc);
       }
 
@@ -362,6 +394,118 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
     dir.close();
   }
 
+  public void testSmallMergesDonNotGetThreads() throws IOException {
+    Directory dir = newDirectory();
+    IndexWriterConfig iwc = new IndexWriterConfig(new MockAnalyzer(random()));
+    iwc.setMaxBufferedDocs(2);
+    iwc.setMergeScheduler(
+        new ConcurrentMergeScheduler() {
+          @Override
+          protected void doMerge(MergeSource mergeSource, MergePolicy.OneMerge merge)
+              throws IOException {
+            assertTrue(this.getIntraMergeExecutor(merge) instanceof SameThreadExecutorService);
+            super.doMerge(mergeSource, merge);
+          }
+        });
+    IndexWriter w = new IndexWriter(dir, iwc);
+    for (int i = 0; i < 10; i++) {
+      Document doc = new Document();
+      doc.add(new StringField("id", "" + i, Field.Store.NO));
+      w.addDocument(doc);
+    }
+    w.forceMerge(1);
+    w.close();
+    dir.close();
+  }
+
+  @SuppressForbidden(reason = "Thread sleep")
+  public void testIntraMergeThreadPoolIsLimitedByMaxThreads() throws IOException {
+    ConcurrentMergeScheduler mergeScheduler = new ConcurrentMergeScheduler();
+    MergeScheduler.MergeSource mergeSource =
+        new MergeScheduler.MergeSource() {
+          @Override
+          public MergePolicy.OneMerge getNextMerge() {
+            fail("should not be called");
+            return null;
+          }
+
+          @Override
+          public void onMergeFinished(MergePolicy.OneMerge merge) {
+            fail("should not be called");
+          }
+
+          @Override
+          public boolean hasPendingMerges() {
+            fail("should not be called");
+            return false;
+          }
+
+          @Override
+          public void merge(MergePolicy.OneMerge merge) throws IOException {
+            fail("should not be called");
+          }
+        };
+    try (Directory dir = newDirectory();
+        mergeScheduler) {
+      MergePolicy.OneMerge merge =
+          new MergePolicy.OneMerge(
+              List.of(
+                  new SegmentCommitInfo(
+                      new SegmentInfo(
+                          dir,
+                          Version.LATEST,
+                          null,
+                          "test",
+                          0,
+                          false,
+                          false,
+                          Codec.getDefault(),
+                          Collections.emptyMap(),
+                          StringHelper.randomId(),
+                          new HashMap<>(),
+                          null),
+                      0,
+                      0,
+                      0,
+                      0,
+                      0,
+                      new byte[16])));
+      mergeScheduler.initialize(InfoStream.NO_OUTPUT, dir);
+      mergeScheduler.setMaxMergesAndThreads(6, 6);
+      Executor executor = mergeScheduler.intraMergeExecutor;
+      AtomicInteger threadsExecutedOnPool = new AtomicInteger();
+      AtomicInteger threadsExecutedOnSelf = new AtomicInteger();
+      for (int i = 0; i < 4; i++) {
+        mergeScheduler.mergeThreads.add(
+            mergeScheduler.new MergeThread(mergeSource, merge) {
+              @Override
+              @SuppressForbidden(reason = "Thread sleep")
+              public void run() {
+                executor.execute(
+                    () -> {
+                      if (Thread.currentThread() == this) {
+                        threadsExecutedOnSelf.incrementAndGet();
+                      } else {
+                        threadsExecutedOnPool.incrementAndGet();
+                      }
+                      try {
+                        Thread.sleep(100);
+                      } catch (InterruptedException e) {
+                        throw new RuntimeException(e);
+                      }
+                    });
+              }
+            });
+      }
+      for (ConcurrentMergeScheduler.MergeThread thread : mergeScheduler.mergeThreads) {
+        thread.start();
+      }
+      mergeScheduler.sync();
+      assertEquals(3, threadsExecutedOnSelf.get());
+      assertEquals(1, threadsExecutedOnPool.get());
+    }
+  }
+
   private static class TrackingCMS extends ConcurrentMergeScheduler {
     long totMergedBytes;
     CountDownLatch atLeastOneMerge;
@@ -660,6 +804,7 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
     iwc.setMaxBufferedDocs(2);
     LogMergePolicy lmp = newLogMergePolicy();
     lmp.setMergeFactor(2);
+    lmp.setTargetSearchConcurrency(1);
     iwc.setMergePolicy(lmp);
 
     IndexWriter w = new IndexWriter(dir, iwc);
@@ -784,6 +929,7 @@ public class TestConcurrentMergeScheduler extends LuceneTestCase {
     IndexWriterConfig iwc = newIndexWriterConfig(new MockAnalyzer(random()));
     iwc.setMergePolicy(NoMergePolicy.INSTANCE);
     iwc.setMaxBufferedDocs(2);
+    iwc.setUseCompoundFile(true); // reduce open files
     IndexWriter w = new IndexWriter(dir, iwc);
     int numDocs = TEST_NIGHTLY ? 1000 : 100;
     for (int i = 0; i < numDocs; i++) {
