@@ -755,8 +755,9 @@ public class TestNumericDocValuesUpdates extends LuceneTestCase {
       if (VERBOSE) {
         System.out.println("TEST: maxDoc=" + r.maxDoc());
       }
+      StoredFields storedFields = r.storedFields();
       for (int i = 0; i < r.maxDoc(); i++) {
-        Document rdoc = r.document(i);
+        Document rdoc = storedFields.document(i);
         assertEquals(i, ndv.nextDoc());
         assertEquals("docid=" + i + " has wrong ndv value; doc=" + rdoc, value, ndv.longValue());
       }
@@ -904,11 +905,12 @@ public class TestNumericDocValuesUpdates extends LuceneTestCase {
           NumericDocValues values = leafReader.getNumericDocValues("number");
           NumericDocValues sortValues = leafReader.getNumericDocValues("sort");
           Bits liveDocs = leafReader.getLiveDocs();
+          StoredFields storedFields = leafReader.storedFields();
 
           long lastSortValue = Long.MIN_VALUE;
           for (int i = 0; i < leafReader.maxDoc(); i++) {
 
-            Document doc = leafReader.document(i);
+            Document doc = storedFields.document(i);
             OneSortDoc sortDoc = docs.get(Integer.parseInt(doc.get("id")));
 
             assertEquals(i, values.nextDoc());
@@ -1027,6 +1029,7 @@ public class TestNumericDocValuesUpdates extends LuceneTestCase {
       for (LeafReaderContext context : reader.leaves()) {
         LeafReader r = context.reader();
         Bits liveDocs = r.getLiveDocs();
+        StoredFields storedFields = r.storedFields();
         for (int field = 0; field < fieldValues.length; field++) {
           String f = "f" + field;
           NumericDocValues ndv = r.getNumericDocValues(f);
@@ -1039,13 +1042,13 @@ public class TestNumericDocValuesUpdates extends LuceneTestCase {
                   "invalid value for docID="
                       + doc
                       + " id="
-                      + r.document(doc).get("id")
+                      + storedFields.document(doc).get("id")
                       + ", field="
                       + f
                       + ", reader="
                       + r
                       + " doc="
-                      + r.document(doc),
+                      + storedFields.document(doc),
                   fieldValues[field],
                   ndv.longValue());
             }
