@@ -18,6 +18,9 @@ package org.apache.lucene.search;
 
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.List;
+import java.util.Objects;
+import java.util.stream.Collectors;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field.Store;
 import org.apache.lucene.document.StringField;
@@ -108,20 +111,21 @@ public class TestWANDScorer extends LuceneTestCase {
     IndexSearcher searcher = newSearcher(reader);
 
     Query query =
-        new BooleanQuery.Builder()
-            .add(
-                new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                Occur.SHOULD)
-            .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-            .add(
-                new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
-                Occur.SHOULD)
-            .build();
+        new WANDScorerQuery(
+            new BooleanQuery.Builder()
+                .add(
+                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                    Occur.SHOULD)
+                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                .add(
+                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                    Occur.SHOULD)
+                .build());
 
-    Scorer scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    Weight weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+    ScorerSupplier ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    Scorer scorer = ss.get(Long.MAX_VALUE);
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -140,10 +144,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
     scorer.setMinCompetitiveScore(4);
 
     assertEquals(3, scorer.iterator().nextDoc());
@@ -154,10 +157,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -170,21 +172,24 @@ public class TestWANDScorer extends LuceneTestCase {
     query =
         new BooleanQuery.Builder()
             .add(
-                new BooleanQuery.Builder()
-                    .add(
-                        new BoostQuery(
-                            new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                        Occur.SHOULD)
-                    .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-                    .build(),
+                new WANDScorerQuery(
+                    new BooleanQuery.Builder()
+                        .add(
+                            new BoostQuery(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                            Occur.SHOULD)
+                        .add(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                            Occur.SHOULD)
+                        .build()),
                 Occur.MUST)
             .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
             .build();
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
 
     assertEquals(3, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -194,10 +199,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
 
     scorer.setMinCompetitiveScore(2);
 
@@ -210,16 +214,24 @@ public class TestWANDScorer extends LuceneTestCase {
     query =
         new BooleanQuery.Builder()
             .add(
-                new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                Occur.SHOULD)
-            .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                new WANDScorerQuery(
+                    new BooleanQuery.Builder()
+                        .add(
+                            new BoostQuery(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                            Occur.SHOULD)
+                        .add(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                            Occur.SHOULD)
+                        .build()),
+                Occur.MUST)
             .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
             .build();
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -232,10 +244,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer =
-        searcher
-            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-            .scorer(searcher.getIndexReader().leaves().get(0));
+    ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+    ss.setTopLevelScoringClause();
+    scorer = ss.get(Long.MAX_VALUE);
 
     scorer.setMinCompetitiveScore(3);
 
@@ -275,21 +286,24 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                        Occur.SHOULD)
+                    .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                        Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
-        Scorer scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        Weight weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+        ScorerSupplier ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        Scorer scorer = ss.get(Long.MAX_VALUE);
 
         assertEquals(0, scorer.iterator().nextDoc());
         assertEquals(2 + 1, scorer.score(), 0);
@@ -302,10 +316,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
         assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-        scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        scorer = ss.get(Long.MAX_VALUE);
         scorer.setMinCompetitiveScore(4);
 
         assertEquals(3, scorer.iterator().nextDoc());
@@ -316,10 +329,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
         assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-        scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        scorer = ss.get(Long.MAX_VALUE);
 
         assertEquals(0, scorer.iterator().nextDoc());
         assertEquals(2 + 1, scorer.score(), 0);
@@ -360,17 +372,18 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(new TermQuery(new Term("foo", "A")), Occur.SHOULD)
-                .add(new TermQuery(new Term("foo", "B")), Occur.SHOULD)
-                .add(new TermQuery(new Term("foo", "C")), Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(new TermQuery(new Term("foo", "A")), Occur.SHOULD)
+                    .add(new TermQuery(new Term("foo", "B")), Occur.SHOULD)
+                    .add(new TermQuery(new Term("foo", "C")), Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
-        Scorer scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        Weight weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+        ScorerSupplier ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        Scorer scorer = ss.get(Long.MAX_VALUE);
 
         assertEquals(0, scorer.iterator().nextDoc());
         scorer.setMinCompetitiveScore(scorer.score());
@@ -407,16 +420,19 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                        Occur.SHOULD)
+                    .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                        Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
         Scorer scorer =
             searcher
@@ -460,28 +476,29 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BooleanQuery.Builder()
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                            Occur.SHOULD)
-                        .add(
-                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
-                            Occur.SHOULD)
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                            Occur.SHOULD)
-                        .setMinimumNumberShouldMatch(2)
-                        .build(),
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
                     Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
                 .build();
 
-        Scorer scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        Weight weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+        ScorerSupplier ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        Scorer scorer = ss.get(Long.MAX_VALUE);
 
         assertEquals(1, scorer.iterator().nextDoc());
         assertEquals(2 + 4, scorer.score(), 0);
@@ -491,10 +508,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
         assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-        scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        scorer = ss.get(Long.MAX_VALUE);
 
         scorer.setMinCompetitiveScore(2 + 1 + 4);
 
@@ -536,20 +552,21 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BooleanQuery.Builder()
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                            Occur.SHOULD)
-                        .add(
-                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
-                            Occur.SHOULD)
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                            Occur.SHOULD)
-                        .setMinimumNumberShouldMatch(2)
-                        .build(),
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
                     Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
                 .build();
@@ -595,20 +612,27 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
+                    Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
                 .build();
 
-        Scorer scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        Weight weight = searcher.createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1);
+        Scorer scorer = weight.scorer(searcher.getIndexReader().leaves().get(0));
 
         assertEquals(0, scorer.iterator().nextDoc());
         assertEquals(2 + 1, scorer.score(), 0);
@@ -618,10 +642,9 @@ public class TestWANDScorer extends LuceneTestCase {
 
         assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-        scorer =
-            searcher
-                .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-                .scorer(searcher.getIndexReader().leaves().get(0));
+        ScorerSupplier ss = weight.scorerSupplier(searcher.getIndexReader().leaves().get(0));
+        ss.setTopLevelScoringClause();
+        scorer = ss.get(Long.MAX_VALUE);
 
         scorer.setMinCompetitiveScore(4);
 
@@ -663,14 +686,23 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
+                    Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
                 .build();
 
         Scorer scorer =
@@ -700,7 +732,10 @@ public class TestWANDScorer extends LuceneTestCase {
     }
     IndexReader reader = DirectoryReader.open(w);
     w.close();
-    IndexSearcher searcher = newSearcher(reader);
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
 
     for (int iter = 0; iter < 100; ++iter) {
       int start = random().nextInt(10);
@@ -710,7 +745,7 @@ public class TestWANDScorer extends LuceneTestCase {
         builder.add(
             maybeWrap(new TermQuery(new Term("foo", Integer.toString(start + i)))), Occur.SHOULD);
       }
-      Query query = builder.build();
+      Query query = new WANDScorerQuery(builder.build());
 
       CheckHits.checkTopScores(random(), query, searcher);
 
@@ -743,7 +778,10 @@ public class TestWANDScorer extends LuceneTestCase {
     }
     IndexReader reader = DirectoryReader.open(w);
     w.close();
-    IndexSearcher searcher = newSearcher(reader);
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
 
     for (int iter = 0; iter < 100; ++iter) {
       int start = random().nextInt(10);
@@ -758,7 +796,7 @@ public class TestWANDScorer extends LuceneTestCase {
                     0f)),
             Occur.SHOULD);
       }
-      Query query = builder.build();
+      Query query = new WANDScorerQuery(builder.build());
 
       CheckHits.checkTopScores(random(), query, searcher);
 
@@ -800,7 +838,10 @@ public class TestWANDScorer extends LuceneTestCase {
     }
     IndexReader reader = DirectoryReader.open(w);
     w.close();
-    IndexSearcher searcher = newSearcher(reader);
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
 
     for (int iter = 0; iter < 100; ++iter) {
       int start = random().nextInt(10);
@@ -815,7 +856,7 @@ public class TestWANDScorer extends LuceneTestCase {
         }
         builder.add(query, Occur.SHOULD);
       }
-      Query query = builder.build();
+      Query query = new WANDScorerQuery(builder.build());
 
       CheckHits.checkTopScores(random(), query, searcher);
 
@@ -899,12 +940,12 @@ public class TestWANDScorer extends LuceneTestCase {
     }
 
     @Override
-    public Query rewrite(IndexReader reader) throws IOException {
-      Query rewritten = query.rewrite(reader);
+    public Query rewrite(IndexSearcher indexSearcher) throws IOException {
+      Query rewritten = query.rewrite(indexSearcher);
       if (rewritten != query) {
         return new MaxScoreWrapperQuery(rewritten, maxRange, maxScore);
       }
-      return super.rewrite(reader);
+      return super.rewrite(indexSearcher);
     }
 
     @Override
@@ -947,4 +988,85 @@ public class TestWANDScorer extends LuceneTestCase {
       };
     }
   }
+
+  private static class WANDScorerQuery extends Query {
+    private final BooleanQuery query;
+
+    private WANDScorerQuery(BooleanQuery query) {
+      assert query.clauses().size() == query.getClauses(Occur.SHOULD).size()
+          : "This test utility query is only used to create WANDScorer for disjunctions.";
+      this.query = query;
+    }
+
+    @Override
+    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
+        throws IOException {
+      return new Weight(query) {
+
+        @Override
+        public Explanation explain(LeafReaderContext context, int doc) throws IOException {
+          // no-ops
+          return null;
+        }
+
+        @Override
+        public Scorer scorer(LeafReaderContext context) throws IOException {
+          BooleanWeight weight = (BooleanWeight) query.createWeight(searcher, scoreMode, boost);
+          List<Scorer> optionalScorers =
+              weight.weightedClauses.stream()
+                  .map(wc -> wc.weight)
+                  .map(
+                      w -> {
+                        try {
+                          return w.scorerSupplier(context);
+                        } catch (IOException e) {
+                          throw new AssertionError(e);
+                        }
+                      })
+                  .filter(Objects::nonNull)
+                  .map(
+                      ss -> {
+                        try {
+                          return ss.get(Long.MAX_VALUE);
+                        } catch (IOException e) {
+                          throw new AssertionError(e);
+                        }
+                      })
+                  .collect(Collectors.toList());
+
+          if (optionalScorers.size() > 0) {
+            return new WANDScorer(
+                weight, optionalScorers, query.getMinimumNumberShouldMatch(), scoreMode);
+          } else {
+            return weight.scorer(context);
+          }
+        }
+
+        @Override
+        public boolean isCacheable(LeafReaderContext ctx) {
+          return false;
+        }
+      };
+    }
+
+    @Override
+    public String toString(String field) {
+      return "WANDScorerQuery";
+    }
+
+    @Override
+    public void visit(QueryVisitor visitor) {
+      // no-ops
+    }
+
+    @Override
+    public boolean equals(Object other) {
+      return sameClassAs(other) && query.equals(((TestWANDScorer.WANDScorerQuery) other).query);
+    }
+
+    @Override
+    public int hashCode() {
+      return 31 * classHash() + query.hashCode();
+    }
+  }
 }
