@@ -219,16 +219,30 @@ public class DrillSideways {
       // There are no drill-down dims, so there is no
       // drill-sideways to compute:
       FacetsCollector drillDownCollector = createDrillDownFacetsCollector();
+      Collector mainCollector;
       if (drillDownCollector != null) {
-        // Make sure we still populate a facet collector for the base query if desired:
-        searcher.search(query, MultiCollector.wrap(hitCollector, drillDownCollector));
+        mainCollector = MultiCollector.wrap(hitCollector, drillDownCollector);
       } else {
-        searcher.search(query, hitCollector);
+        mainCollector = hitCollector;
       }
+      searcher.search(query, mainCollector);
       return new DrillSidewaysResult(
           buildFacetsResult(drillDownCollector, null, null), null, drillDownCollector, null, null);
     }
 
+    // Note: it's strange that we delegate to createDrillDownFacetsCollectorManager here instead of
+    // createDrillDownFacetsCollector. We should probably be using createDrillDownFacetsCollector
+    // here instead, but this is pre-existing behavior in a deprecated method, so we're choosing to
+    // keep it as-is. See conversation in
+    // https://github.com/apache/lucene/pull/13716#discussion_r1744486418.
+    FacetsCollector drillDownCollector = createDrillDownFacetsCollectorManager().newCollector();
+    Collector mainCollector;
+    if (drillDownCollector != null) {
+      mainCollector = MultiCollector.wrap(hitCollector, drillDownCollector);
+    } else {
+      mainCollector = hitCollector;
+    }
+
     Query baseQuery = query.getBaseQuery();
     if (baseQuery == null) {
       // TODO: we could optimize this pure-browse case by
@@ -239,30 +253,17 @@ public class DrillSideways {
 
     int numDims = drillDownDims.size();
 
-    FacetsCollectorManager drillDownCollectorManager = createDrillDownFacetsCollectorManager();
-
-    FacetsCollectorManager[] drillSidewaysFacetsCollectorManagers =
-        new FacetsCollectorManager[numDims];
+    final List<CollectorManager<FacetsCollector, FacetsCollector>> drillSidewaysCollectorManagers =
+        new ArrayList<>(numDims);
     for (int i = 0; i < numDims; i++) {
-      drillSidewaysFacetsCollectorManagers[i] = new FacetsCollectorManager();
+      drillSidewaysCollectorManagers.add(createDrillSidewaysFacetsCollectorManager());
     }
 
-    DrillSidewaysQuery dsq =
-        new DrillSidewaysQuery(
-            baseQuery,
-            drillDownCollectorManager,
-            drillSidewaysFacetsCollectorManagers,
-            drillDownQueries,
-            scoreSubDocsAtOnce());
+    DrillSidewaysQuery<FacetsCollector, ?> dsq =
+        new DrillSidewaysQuery<>(
+            baseQuery, drillSidewaysCollectorManagers, drillDownQueries, scoreSubDocsAtOnce());
 
-    searcher.search(dsq, hitCollector);
-
-    FacetsCollector drillDownCollector;
-    if (drillDownCollectorManager != null) {
-      drillDownCollector = drillDownCollectorManager.reduce(dsq.managedDrillDownCollectors);
-    } else {
-      drillDownCollector = null;
-    }
+    searcher.search(dsq, mainCollector);
 
     FacetsCollector[] drillSidewaysCollectors = new FacetsCollector[numDims];
     int numSlices = dsq.managedDrillSidewaysCollectors.size();
@@ -271,14 +272,14 @@ public class DrillSideways {
       List<FacetsCollector> facetsCollectorsForDim = new ArrayList<>(numSlices);
 
       for (int slice = 0; slice < numSlices; slice++) {
-        facetsCollectorsForDim.add(dsq.managedDrillSidewaysCollectors.get(slice)[dim]);
+        facetsCollectorsForDim.add(dsq.managedDrillSidewaysCollectors.get(slice).get(dim));
       }
 
       drillSidewaysCollectors[dim] =
-          drillSidewaysFacetsCollectorManagers[dim].reduce(facetsCollectorsForDim);
+          drillSidewaysCollectorManagers.get(dim).reduce(facetsCollectorsForDim);
     }
 
-    String[] drillSidewaysDims = drillDownDims.keySet().toArray(new String[0]);
+    String[] drillSidewaysDims = query.getDims().keySet().toArray(new String[0]);
 
     return new DrillSidewaysResult(
         buildFacetsResult(drillDownCollector, drillSidewaysCollectors, drillSidewaysDims),
@@ -405,35 +406,21 @@ public class DrillSideways {
     }
   }
 
-  private static class CallableCollector implements Callable<CallableResult> {
-
-    private final int pos;
+  private static class CallableCollector<R> implements Callable<R> {
     private final IndexSearcher searcher;
     private final Query query;
-    private final CollectorManager<?, ?> collectorManager;
+    private final CollectorManager<?, R> collectorManager;
 
     private CallableCollector(
-        int pos, IndexSearcher searcher, Query query, CollectorManager<?, ?> collectorManager) {
-      this.pos = pos;
+        IndexSearcher searcher, Query query, CollectorManager<?, R> collectorManager) {
       this.searcher = searcher;
       this.query = query;
       this.collectorManager = collectorManager;
     }
 
     @Override
-    public CallableResult call() throws Exception {
-      return new CallableResult(pos, searcher.search(query, collectorManager));
-    }
-  }
-
-  private static class CallableResult {
-
-    private final int pos;
-    private final Object result;
-
-    private CallableResult(int pos, Object result) {
-      this.pos = pos;
-      this.result = result;
+    public R call() throws Exception {
+      return searcher.search(query, collectorManager);
     }
   }
 
@@ -454,16 +441,138 @@ public class DrillSideways {
   public <R> ConcurrentDrillSidewaysResult<R> search(
       final DrillDownQuery query, final CollectorManager<?, R> hitCollectorManager)
       throws IOException {
-    if (executor != null) {
-      return searchConcurrently(query, hitCollectorManager);
+    // Main query
+    FacetsCollectorManager drillDownFacetsCollectorManager =
+        createDrillDownFacetsCollectorManager();
+    final CollectorManager<?, ?> mainCollectorManager;
+    if (drillDownFacetsCollectorManager != null) {
+      // Make sure we populate a facet collector corresponding to the base query if desired:
+      mainCollectorManager =
+          new MultiCollectorManager(drillDownFacetsCollectorManager, hitCollectorManager);
     } else {
-      return searchSequentially(query, hitCollectorManager);
+      mainCollectorManager = hitCollectorManager;
+    }
+    // Drill sideways dimensions
+    final List<FacetsCollectorManager> drillSidewaysCollectorManagers;
+    if (query.getDims().isEmpty() == false) {
+      drillSidewaysCollectorManagers = new ArrayList<>(query.getDims().size());
+      for (int i = 0; i < query.getDims().size(); i++) {
+        drillSidewaysCollectorManagers.add(createDrillSidewaysFacetsCollectorManager());
+      }
+    } else {
+      drillSidewaysCollectorManagers = null;
+    }
+    // Execute query
+    final Result<?, FacetsCollector> result;
+    if (executor != null) {
+      result = searchConcurrently(query, mainCollectorManager, drillSidewaysCollectorManagers);
+    } else {
+      result = searchSequentially(query, mainCollectorManager, drillSidewaysCollectorManagers);
+    }
+
+    // Collect results
+    final FacetsCollector facetsCollectorResult;
+    final R hitCollectorResult;
+    if (drillDownFacetsCollectorManager != null) {
+      // drill down collected using MultiCollector
+      // Extract the results:
+      Object[] drillDownResult = (Object[]) result.drillDownResult;
+      facetsCollectorResult = (FacetsCollector) drillDownResult[0];
+      hitCollectorResult = (R) drillDownResult[1];
+    } else {
+      facetsCollectorResult = null;
+      hitCollectorResult = (R) result.drillDownResult;
+    }
+
+    // Getting results for drill sideways dimensions (if any)
+    final String[] drillSidewaysDims;
+    final FacetsCollector[] drillSidewaysCollectors;
+    if (query.getDims().isEmpty() == false) {
+      drillSidewaysDims = query.getDims().keySet().toArray(new String[0]);
+      int numDims = query.getDims().size();
+      assert drillSidewaysCollectorManagers != null;
+      assert drillSidewaysCollectorManagers.size() == numDims;
+      drillSidewaysCollectors = new FacetsCollector[numDims];
+      for (int dim = 0; dim < numDims; dim++) {
+        drillSidewaysCollectors[dim] = result.drillSidewaysResults.get(dim);
+      }
+    } else {
+      drillSidewaysDims = null;
+      drillSidewaysCollectors = null;
+    }
+
+    return new ConcurrentDrillSidewaysResult<>(
+        buildFacetsResult(facetsCollectorResult, drillSidewaysCollectors, drillSidewaysDims),
+        null,
+        hitCollectorResult,
+        facetsCollectorResult,
+        drillSidewaysCollectors,
+        drillSidewaysDims);
+  }
+
+  /**
+   * Search using DrillDownQuery with custom collectors. This method can be used with any {@link
+   * CollectorManager}s.
+   *
+   * <p>Note: Use {@link MultiCollectorManager} to collect both hits and facets for the entire query
+   * and/or for drill-sideways dimensions. You can also use it to wrap different types of {@link
+   * CollectorManager} for drill-sideways dimensions.
+   */
+  public <C extends Collector, T, K extends Collector, R> Result<T, R> search(
+      DrillDownQuery query,
+      CollectorManager<C, T> drillDownCollectorManager,
+      List<? extends CollectorManager<K, R>> drillSidewaysCollectorManagers)
+      throws IOException {
+    if (drillDownCollectorManager == null) {
+      throw new IllegalArgumentException(
+          "This search method requires client to provide drill down collector manager");
+    }
+    if (drillSidewaysCollectorManagers == null) {
+      if (query.getDims().isEmpty() == false) {
+        throw new IllegalArgumentException(
+            "The query requires not null drillSidewaysCollectorManagers");
+      }
+    } else if (drillSidewaysCollectorManagers.size() != query.getDims().size()) {
+      throw new IllegalArgumentException(
+          "drillSidewaysCollectorManagers size must be equal to number of dimensions in the query.");
+    }
+    if (executor != null) {
+      return searchConcurrently(query, drillDownCollectorManager, drillSidewaysCollectorManagers);
+    } else {
+      return searchSequentially(query, drillDownCollectorManager, drillSidewaysCollectorManagers);
     }
   }
 
-  @SuppressWarnings("unchecked")
-  private <R> ConcurrentDrillSidewaysResult<R> searchSequentially(
-      final DrillDownQuery query, final CollectorManager<?, R> hitCollectorManager)
+  /**
+   * {@link #search(DrillDownQuery, CollectorManager, List)} result. It doesn't depend on {@link
+   * Facets} to allow users to use any type of {@link CollectorManager} for drill-down or
+   * drill-sideways dimension.
+   */
+  public static final class Result<T, R> {
+    private final T drillDownResult;
+    private final List<R> drillSidewaysResults;
+
+    /** sole ctor */
+    public Result(T drillDownResult, List<R> drillSidewaysResults) {
+      this.drillDownResult = drillDownResult;
+      this.drillSidewaysResults = drillSidewaysResults;
+    }
+
+    /** result from drill down (main) {@link CollectorManager} */
+    public T drillDownResult() {
+      return drillDownResult;
+    }
+
+    /** results from drill sideways {@link CollectorManager}s */
+    public List<R> drillSidewaysResults() {
+      return drillSidewaysResults;
+    }
+  }
+
+  private <C extends Collector, T, K extends Collector, R> Result<T, R> searchSequentially(
+      final DrillDownQuery query,
+      final CollectorManager<C, T> drillDownCollectorManager,
+      final List<? extends CollectorManager<K, R>> drillSidewaysCollectorManagers)
       throws IOException {
 
     Map<String, Integer> drillDownDims = query.getDims();
@@ -471,28 +580,7 @@ public class DrillSideways {
     if (drillDownDims.isEmpty()) {
       // There are no drill-down dims, so there is no
       // drill-sideways to compute:
-      FacetsCollectorManager drillDownCollectorManager = createDrillDownFacetsCollectorManager();
-      FacetsCollector mainFacetsCollector;
-      R collectorResult;
-      if (drillDownCollectorManager != null) {
-        Object[] mainResults =
-            searcher.search(
-                query, new MultiCollectorManager(drillDownCollectorManager, hitCollectorManager));
-        // Extract the results:
-        mainFacetsCollector = (FacetsCollector) mainResults[0];
-        collectorResult = (R) mainResults[1];
-      } else {
-        mainFacetsCollector = null;
-        collectorResult = searcher.search(query, hitCollectorManager);
-      }
-
-      return new ConcurrentDrillSidewaysResult<>(
-          buildFacetsResult(mainFacetsCollector, null, null),
-          null,
-          collectorResult,
-          mainFacetsCollector,
-          null,
-          null);
+      return new Result<>(searcher.search(query, drillDownCollectorManager), null);
     }
 
     Query baseQuery = query.getBaseQuery();
@@ -503,130 +591,65 @@ public class DrillSideways {
     }
     Query[] drillDownQueries = query.getDrillDownQueries();
 
-    int numDims = drillDownDims.size();
+    DrillSidewaysQuery<K, R> dsq =
+        new DrillSidewaysQuery<>(
+            baseQuery, drillSidewaysCollectorManagers, drillDownQueries, scoreSubDocsAtOnce());
 
-    FacetsCollectorManager drillDownCollectorManager = createDrillDownFacetsCollectorManager();
-
-    FacetsCollectorManager[] drillSidewaysFacetsCollectorManagers =
-        new FacetsCollectorManager[numDims];
-    for (int i = 0; i < numDims; i++) {
-      drillSidewaysFacetsCollectorManagers[i] = createDrillSidewaysFacetsCollectorManager();
-    }
-
-    DrillSidewaysQuery dsq =
-        new DrillSidewaysQuery(
-            baseQuery,
-            drillDownCollectorManager,
-            drillSidewaysFacetsCollectorManagers,
-            drillDownQueries,
-            scoreSubDocsAtOnce());
-
-    R collectorResult = searcher.search(dsq, hitCollectorManager);
-
-    FacetsCollector drillDownCollector;
-    if (drillDownCollectorManager != null) {
-      drillDownCollector = drillDownCollectorManager.reduce(dsq.managedDrillDownCollectors);
-    } else {
-      drillDownCollector = null;
-    }
-
-    FacetsCollector[] drillSidewaysCollectors = new FacetsCollector[numDims];
+    T collectorResult = searcher.search(dsq, drillDownCollectorManager);
+    List<R> drillSidewaysResults = new ArrayList<>(drillDownDims.size());
+    assert drillSidewaysCollectorManagers != null
+        : "Case without drill sideways dimensions is handled above";
     int numSlices = dsq.managedDrillSidewaysCollectors.size();
-
-    for (int dim = 0; dim < numDims; dim++) {
-      List<FacetsCollector> facetsCollectorsForDim = new ArrayList<>(numSlices);
-
+    for (int dim = 0; dim < drillDownDims.size(); dim++) {
+      List<K> collectorsForDim = new ArrayList<>(numSlices);
       for (int slice = 0; slice < numSlices; slice++) {
-        facetsCollectorsForDim.add(dsq.managedDrillSidewaysCollectors.get(slice)[dim]);
+        collectorsForDim.add(dsq.managedDrillSidewaysCollectors.get(slice).get(dim));
       }
-
-      drillSidewaysCollectors[dim] =
-          drillSidewaysFacetsCollectorManagers[dim].reduce(facetsCollectorsForDim);
+      drillSidewaysResults.add(
+          dim, drillSidewaysCollectorManagers.get(dim).reduce(collectorsForDim));
     }
-
-    String[] drillSidewaysDims = drillDownDims.keySet().toArray(new String[0]);
-
-    return new ConcurrentDrillSidewaysResult<>(
-        buildFacetsResult(drillDownCollector, drillSidewaysCollectors, drillSidewaysDims),
-        null,
-        collectorResult,
-        drillDownCollector,
-        drillSidewaysCollectors,
-        drillSidewaysDims);
+    return new Result<>(collectorResult, drillSidewaysResults);
   }
 
-  @SuppressWarnings("unchecked")
-  private <R> ConcurrentDrillSidewaysResult<R> searchConcurrently(
-      final DrillDownQuery query, final CollectorManager<?, R> hitCollectorManager)
-      throws IOException {
+  private <C extends Collector, T, K extends Collector, R> Result<T, R> searchConcurrently(
+      final DrillDownQuery query,
+      final CollectorManager<C, T> drillDownCollectorManager,
+      final List<? extends CollectorManager<K, R>> drillSidewaysCollectorManagers) {
 
     final Map<String, Integer> drillDownDims = query.getDims();
-    final List<CallableCollector> callableCollectors = new ArrayList<>(drillDownDims.size() + 1);
+    final CallableCollector<T> drillDownCallableCollector =
+        new CallableCollector<>(searcher, query, drillDownCollectorManager);
+    final List<CallableCollector<R>> drillSidewaysCallableCollectors =
+        new ArrayList<>(drillDownDims.size());
 
-    // Add the main DrillDownQuery
-    FacetsCollectorManager drillDownFacetsCollectorManager =
-        createDrillDownFacetsCollectorManager();
-    CollectorManager<?, ?> mainCollectorManager;
-    if (drillDownFacetsCollectorManager != null) {
-      // Make sure we populate a facet collector corresponding to the base query if desired:
-      mainCollectorManager =
-          new MultiCollectorManager(drillDownFacetsCollectorManager, hitCollectorManager);
-    } else {
-      mainCollectorManager = hitCollectorManager;
-    }
-    callableCollectors.add(new CallableCollector(-1, searcher, query, mainCollectorManager));
     int i = 0;
     final Query[] filters = query.getDrillDownQueries();
-    for (String dim : drillDownDims.keySet())
-      callableCollectors.add(
-          new CallableCollector(
-              i++,
+    for (String dim : drillDownDims.keySet()) {
+      drillSidewaysCallableCollectors.add(
+          new CallableCollector<>(
               searcher,
               getDrillDownQuery(query, filters, dim),
-              createDrillSidewaysFacetsCollectorManager()));
-
-    final FacetsCollector mainFacetsCollector;
-    final FacetsCollector[] facetsCollectors = new FacetsCollector[drillDownDims.size()];
-    final R collectorResult;
+              drillSidewaysCollectorManagers.get(i)));
+      i++;
+    }
 
     try {
-      // Run the query pool
-      final List<Future<CallableResult>> futures = executor.invokeAll(callableCollectors);
+      final Future<T> drillDownFuture = executor.submit(drillDownCallableCollector);
+      final List<Future<R>> drillSidewaysFutures =
+          executor.invokeAll(drillSidewaysCallableCollectors);
 
-      // Extract the results
-      if (drillDownFacetsCollectorManager != null) {
-        // If we populated a facets collector for the main query, make sure to unpack it properly
-        final Object[] mainResults = (Object[]) futures.get(0).get().result;
-        mainFacetsCollector = (FacetsCollector) mainResults[0];
-        collectorResult = (R) mainResults[1];
-      } else {
-        mainFacetsCollector = null;
-        collectorResult = (R) futures.get(0).get().result;
-      }
-      for (i = 1; i < futures.size(); i++) {
-        final CallableResult result = futures.get(i).get();
-        facetsCollectors[result.pos] = (FacetsCollector) result.result;
-      }
-      // Fill the null results with the mainFacetsCollector
-      for (i = 0; i < facetsCollectors.length; i++)
-        if (facetsCollectors[i] == null) facetsCollectors[i] = mainFacetsCollector;
+      T collectorResult = drillDownFuture.get();
+      List<R> drillSidewaysResults = new ArrayList<>(drillDownDims.size());
 
+      for (i = 0; i < drillSidewaysFutures.size(); i++) {
+        drillSidewaysResults.add(i, drillSidewaysFutures.get(i).get());
+      }
+      return new Result<>(collectorResult, drillSidewaysResults);
     } catch (InterruptedException e) {
       throw new ThreadInterruptedException(e);
     } catch (ExecutionException e) {
       throw new RuntimeException(e);
     }
-
-    String[] drillSidewaysDims = drillDownDims.keySet().toArray(new String[0]);
-
-    // build the facets and return the result
-    return new ConcurrentDrillSidewaysResult<>(
-        buildFacetsResult(mainFacetsCollector, facetsCollectors, drillSidewaysDims),
-        null,
-        collectorResult,
-        mainFacetsCollector,
-        facetsCollectors,
-        drillSidewaysDims);
   }
 
   /**
