@@ -45,8 +45,6 @@ class DrillSidewaysScorer extends BulkScorer {
 
   // private static boolean DEBUG = false;
 
-  private final LeafCollector drillDownLeafCollector;
-
   private final DocsAndCost[] dims;
 
   // DrillDown DocsEnums:
@@ -68,7 +66,6 @@ class DrillSidewaysScorer extends BulkScorer {
   DrillSidewaysScorer(
       LeafReaderContext context,
       Scorer baseScorer,
-      LeafCollector drillDownLeafCollector,
       DocsAndCost[] dims,
       boolean scoreSubDocsAtOnce) {
     this.dims = dims;
@@ -81,7 +78,6 @@ class DrillSidewaysScorer extends BulkScorer {
     } else {
       this.baseApproximation = baseIterator;
     }
-    this.drillDownLeafCollector = drillDownLeafCollector;
     this.scoreSubDocsAtOnce = scoreSubDocsAtOnce;
   }
 
@@ -708,9 +704,6 @@ class DrillSidewaysScorer extends BulkScorer {
     // }
 
     collector.collect(collectDocID);
-    if (drillDownLeafCollector != null) {
-      drillDownLeafCollector.collect(collectDocID);
-    }
 
     // TODO: we could "fix" faceting of the sideways counts
     // to do this "union" (of the drill down hits) in the
@@ -724,9 +717,6 @@ class DrillSidewaysScorer extends BulkScorer {
 
   private void collectHit(LeafCollector collector, DocsAndCost dim) throws IOException {
     collector.collect(collectDocID);
-    if (drillDownLeafCollector != null) {
-      drillDownLeafCollector.collect(collectDocID);
-    }
 
     // Tally sideways count:
     dim.sidewaysLeafCollector.collect(collectDocID);
@@ -734,9 +724,6 @@ class DrillSidewaysScorer extends BulkScorer {
 
   private void collectHit(LeafCollector collector, List<DocsAndCost> dims) throws IOException {
     collector.collect(collectDocID);
-    if (drillDownLeafCollector != null) {
-      drillDownLeafCollector.collect(collectDocID);
-    }
 
     // Tally sideways counts:
     for (DocsAndCost dim : dims) {
@@ -755,9 +742,6 @@ class DrillSidewaysScorer extends BulkScorer {
     // Note: We _only_ call #finish on the facets collectors we're managing here, but not the
     // "main" collector. This is because IndexSearcher handles calling #finish on the main
     // collector.
-    if (drillDownLeafCollector != null) {
-      drillDownLeafCollector.finish();
-    }
     for (DocsAndCost dim : dims) {
       dim.sidewaysLeafCollector.finish();
     }
@@ -765,9 +749,6 @@ class DrillSidewaysScorer extends BulkScorer {
 
   private void setScorer(LeafCollector mainCollector, Scorable scorer) throws IOException {
     mainCollector.setScorer(scorer);
-    if (drillDownLeafCollector != null) {
-      drillDownLeafCollector.setScorer(scorer);
-    }
     for (DocsAndCost dim : dims) {
       dim.sidewaysLeafCollector.setScorer(scorer);
     }
