@@ -149,7 +149,7 @@ public class TestTaxonomyCombined extends FacetTestCase {
     if (path.length == 0) {
       return "<empty>";
     }
-    return "<" + path.toString() + ">";
+    return "<" + path + ">";
   }
 
   /**
@@ -524,21 +524,10 @@ public class TestTaxonomyCombined extends FacetTestCase {
     }
 
     // check parent of of invalid ordinals:
+    expectThrows(IndexOutOfBoundsException.class, () -> tw.getParent(-1));
     expectThrows(
-        IndexOutOfBoundsException.class,
-        () -> {
-          tw.getParent(-1);
-        });
-    expectThrows(
-        IndexOutOfBoundsException.class,
-        () -> {
-          tw.getParent(TaxonomyReader.INVALID_ORDINAL);
-        });
-    expectThrows(
-        IndexOutOfBoundsException.class,
-        () -> {
-          tw.getParent(tr.getSize());
-        });
+        IndexOutOfBoundsException.class, () -> tw.getParent(TaxonomyReader.INVALID_ORDINAL));
+    expectThrows(IndexOutOfBoundsException.class, () -> tw.getParent(tr.getSize()));
   }
 
   /**
