@@ -146,11 +146,7 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
 
     // test getTopChildren(0, dim)
     final Facets f = facets;
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          f.getTopChildren(0, "Author");
-        });
+    expectThrows(IllegalArgumentException.class, () -> f.getTopChildren(0, "Author"));
 
     taxoReader.close();
     searcher.getIndexReader().close();
@@ -221,11 +217,7 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     List<FacetResult> results = facets.getAllDims(10);
 
     // test getAllDims(0)
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          facets.getAllDims(0);
-        });
+    expectThrows(IllegalArgumentException.class, () -> facets.getAllDims(0));
 
     assertEquals(3, results.size());
     assertEquals(
@@ -250,18 +242,10 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     assertEquals(results, allDimsResults);
 
     // test getTopDims(0, 1)
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          facets.getTopDims(0, 1);
-        });
+    expectThrows(IllegalArgumentException.class, () -> facets.getTopDims(0, 1));
 
     // test getTopDims(1, 0) with topNChildren = 0
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          facets.getTopDims(1, 0);
-        });
+    expectThrows(IllegalArgumentException.class, () -> facets.getTopDims(1, 0));
 
     IOUtils.close(searcher.getIndexReader(), taxoReader, dir, taxoDir);
   }
@@ -313,17 +297,9 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     // test default implementation of getTopDims
     List<FacetResult> topDimsResults = facets.getTopDims(10, 10);
     assertTrue(topDimsResults.isEmpty());
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          facets.getSpecificValue("a");
-        });
+    expectThrows(IllegalArgumentException.class, () -> facets.getSpecificValue("a"));
 
-    expectThrows(
-        IllegalArgumentException.class,
-        () -> {
-          facets.getTopChildren(10, "a");
-        });
+    expectThrows(IllegalArgumentException.class, () -> facets.getTopChildren(10, "a"));
 
     IOUtils.close(searcher.getIndexReader(), taxoReader, dir, taxoDir);
   }
@@ -349,10 +325,13 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     DirectoryReader r = DirectoryReader.open(iw);
     DirectoryTaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
 
-    FacetsCollector fc = new FacetsCollector(true);
     BoostQuery csq = new BoostQuery(new ConstantScoreQuery(new MatchAllDocsQuery()), 2f);
 
-    TopDocs td = FacetsCollector.search(newSearcher(r), csq, 10, fc);
+    FacetsCollectorManager.FacetsResult facetsResult =
+        FacetsCollectorManager.search(newSearcher(r), csq, 10, new FacetsCollectorManager(true));
+    TopDocs td = facetsResult.topDocs();
+    FacetsCollector fc = facetsResult.facetsCollector();
+    assertTrue(fc.getKeepScores());
 
     // Test SUM:
     Facets facets =
@@ -459,11 +438,13 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     DirectoryReader r = DirectoryReader.open(iw);
     DirectoryTaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
 
-    FacetsCollector fc = new FacetsCollector(true);
     // score documents by their 'price' field - makes asserting the correct counts for the
     // categories easier
     Query q = new FunctionQuery(new LongFieldSource("price"));
-    FacetsCollector.search(newSearcher(r), q, 10, fc);
+    FacetsCollector fc =
+        FacetsCollectorManager.search(newSearcher(r), q, 10, new FacetsCollectorManager(true))
+            .facetsCollector();
+    assertTrue(fc.getKeepScores());
 
     // Test SUM:
     Facets facets =
@@ -552,53 +533,44 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
   }
 
   // LUCENE-10495
-  public void testSiblingsLoaded() throws Exception {
-    Directory indexDir = newDirectory();
-    Directory taxoDir = newDirectory();
+  public void testChildrenAndSiblingsLoaded() throws Exception {
+    boolean[] shouldLoad = new boolean[] {false, true};
+    for (boolean load : shouldLoad) {
+      Directory indexDir = newDirectory();
+      Directory taxoDir = newDirectory();
 
-    DirectoryTaxonomyWriter taxoWriter = new DirectoryTaxonomyWriter(taxoDir);
-    IndexWriter iw = new IndexWriter(indexDir, newIndexWriterConfig(new MockAnalyzer(random())));
-    FacetsConfig config = new FacetsConfig();
+      DirectoryTaxonomyWriter taxoWriter = new DirectoryTaxonomyWriter(taxoDir);
+      IndexWriter iw = new IndexWriter(indexDir, newIndexWriterConfig(new MockAnalyzer(random())));
+      FacetsConfig config = new FacetsConfig();
 
-    config.setHierarchical("a", true);
-    config.setMultiValued("a", true);
-    config.setRequireDimCount("a", true);
+      config.setHierarchical("a", true);
+      config.setMultiValued("a", load == false);
+      config.setRequireDimCount("a", true);
 
-    Document doc = new Document();
-    doc.add(new FacetField("a", Integer.toString(2), "1"));
-    iw.addDocument(config.build(taxoWriter, doc));
+      Document doc = new Document();
+      doc.add(new FacetField("a", "1", "2"));
+      iw.addDocument(config.build(taxoWriter, doc));
 
-    DirectoryReader r = DirectoryReader.open(iw);
-    DirectoryTaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
+      DirectoryReader r = DirectoryReader.open(iw);
+      DirectoryTaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
 
-    FacetsCollector sfc =
-        newSearcher(r).search(new MatchAllDocsQuery(), new FacetsCollectorManager());
+      FacetsCollector sfc =
+          newSearcher(r).search(new MatchAllDocsQuery(), new FacetsCollectorManager());
 
-    // Test MAX:
-    Facets facets =
-        new TaxonomyFacetFloatAssociations(
-            taxoReader,
-            config,
-            sfc,
-            AssociationAggregationFunction.MAX,
-            DoubleValuesSource.fromLongField("price"));
+      TaxonomyFacets facets =
+          new TaxonomyFacetFloatAssociations(
+              taxoReader,
+              config,
+              sfc,
+              AssociationAggregationFunction.MAX,
+              DoubleValuesSource.fromLongField("price"));
 
-    assertTrue(((TaxonomyFacets) facets).childrenLoaded());
-    assertFalse(((TaxonomyFacets) facets).siblingsLoaded());
+      assertEquals(load, facets.childrenLoaded());
+      assertEquals(load, facets.siblingsLoaded());
 
-    // Test SUM:
-    facets =
-        new TaxonomyFacetFloatAssociations(
-            taxoReader,
-            config,
-            sfc,
-            AssociationAggregationFunction.SUM,
-            DoubleValuesSource.fromLongField("price"));
-    assertTrue(((TaxonomyFacets) facets).childrenLoaded());
-    assertFalse(((TaxonomyFacets) facets).siblingsLoaded());
-
-    iw.close();
-    IOUtils.close(taxoWriter, taxoReader, taxoDir, r, indexDir);
+      iw.close();
+      IOUtils.close(taxoWriter, taxoReader, taxoDir, r, indexDir);
+    }
   }
 
   public void testCountAndSumScore() throws Exception {
@@ -621,8 +593,11 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     DirectoryReader r = DirectoryReader.open(iw);
     DirectoryTaxonomyReader taxoReader = new DirectoryTaxonomyReader(taxoWriter);
 
-    FacetsCollector fc = new FacetsCollector(true);
-    FacetsCollector.search(newSearcher(r), new MatchAllDocsQuery(), 10, fc);
+    FacetsCollector fc =
+        FacetsCollectorManager.search(
+                newSearcher(r), new MatchAllDocsQuery(), 10, new FacetsCollectorManager(true))
+            .facetsCollector();
+    assertTrue(fc.getKeepScores());
 
     Facets facets1 = getTaxonomyFacetCounts(taxoReader, config, fc);
     Facets facets2;
@@ -690,8 +665,13 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
       if (VERBOSE) {
         System.out.println("\nTEST: iter content=" + searchToken);
       }
-      FacetsCollector fc = new FacetsCollector();
-      FacetsCollector.search(searcher, new TermQuery(new Term("content", searchToken)), 10, fc);
+      FacetsCollector fc =
+          FacetsCollectorManager.search(
+                  searcher,
+                  new TermQuery(new Term("content", searchToken)),
+                  10,
+                  new FacetsCollectorManager())
+              .facetsCollector();
 
       checkResults(
           numDims,
@@ -765,7 +745,7 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
                 "dim" + i,
                 new String[0],
                 aggregatedValue,
-                labelValues.toArray(new LabelAndValue[labelValues.size()]),
+                labelValues.toArray(new LabelAndValue[0]),
                 labelValues.size()));
       }
     }
@@ -795,8 +775,8 @@ public class TestTaxonomyFacetValueSource extends FacetTestCase {
     sortTies(actual);
 
     if (VERBOSE) {
-      System.out.println("expected=\n" + expected.toString());
-      System.out.println("actual=\n" + actual.toString());
+      System.out.println("expected=\n" + expected);
+      System.out.println("actual=\n" + actual);
     }
 
     assertFloatValuesEquals(expected, actual);
