@@ -49,6 +49,7 @@ import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.IndexWriterConfig.OpenMode;
 import org.apache.lucene.index.Term;
+import org.apache.lucene.index.TermVectors;
 import org.apache.lucene.queries.CommonTermsQuery;
 import org.apache.lucene.queries.function.FunctionScoreQuery;
 import org.apache.lucene.queries.payloads.SpanPayloadCheckQuery;
@@ -62,6 +63,7 @@ import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.BooleanQuery;
 import org.apache.lucene.search.ConstantScoreQuery;
 import org.apache.lucene.search.DoubleValuesSource;
+import org.apache.lucene.search.FieldExistsQuery;
 import org.apache.lucene.search.FuzzyQuery;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.MultiPhraseQuery;
@@ -139,8 +141,8 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
   private TokenStream getAnyTokenStream(String fieldName, int docId) throws IOException {
     return TokenSources.getTokenStream(
         fieldName,
-        searcher.getIndexReader().getTermVectors(docId),
-        searcher.doc(docId).get(fieldName),
+        searcher.getIndexReader().termVectors().get(docId),
+        searcher.storedFields().document(docId).get(fieldName),
         analyzer,
         -1);
   }
@@ -156,7 +158,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     Highlighter highlighter = new Highlighter(scorer);
 
     final int docId0 = hits.scoreDocs[0].doc;
-    Document doc = searcher.doc(docId0);
+    Document doc = searcher.storedFields().document(docId0);
     String storedField = doc.get(FIELD_NAME);
 
     TokenStream stream = getAnyTokenStream(FIELD_NAME, docId0);
@@ -180,7 +182,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.scoreDocs.length; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      Document doc = searcher.doc(docId);
+      Document doc = searcher.storedFields().document(docId);
       String storedField = doc.get(FIELD_NAME);
 
       TokenStream stream = getAnyTokenStream(FIELD_NAME, docId);
@@ -208,7 +210,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     Highlighter highlighter = new Highlighter(scorer);
 
     final int docId0 = hits.scoreDocs[0].doc;
-    Document doc = searcher.doc(docId0);
+    Document doc = searcher.storedFields().document(docId0);
     String storedField = doc.get(FIELD_NAME);
 
     TokenStream stream = getAnyTokenStream(FIELD_NAME, docId0);
@@ -220,7 +222,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
         fragment);
 
     final int docId1 = hits.scoreDocs[1].doc;
-    doc = searcher.doc(docId1);
+    doc = searcher.storedFields().document(docId1);
     storedField = doc.get(FIELD_NAME);
 
     stream = getAnyTokenStream(FIELD_NAME, docId1);
@@ -243,12 +245,12 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     TokenStream stream = getAnyTokenStream(FIELD_NAME, 2);
     Fragmenter fragmenter = new SimpleSpanFragmenter(scorer);
     highlighter.setTextFragmenter(fragmenter);
-    String storedField = searcher.doc(2).get(FIELD_NAME);
+    String storedField = searcher.storedFields().document(2).get(FIELD_NAME);
     String fragment = highlighter.getBestFragment(stream, storedField);
     assertEquals("<B>JFK</B> has been shot", fragment);
 
     stream = getAnyTokenStream(FIELD_NAME, 3);
-    storedField = searcher.doc(3).get(FIELD_NAME);
+    storedField = searcher.storedFields().document(3).get(FIELD_NAME);
     fragment = highlighter.getBestFragment(stream, storedField);
     assertEquals("John <B>Kennedy</B> has been shot", fragment);
   }
@@ -259,7 +261,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
         new Query() {
 
           @Override
-          public Query rewrite(IndexReader reader) throws IOException {
+          public Query rewrite(IndexSearcher indexSearcher) throws IOException {
             CommonTermsQuery query = new CommonTermsQuery(Occur.MUST, Occur.SHOULD, 3);
             query.add(new Term(FIELD_NAME, "this")); // stop-word
             query.add(new Term(FIELD_NAME, "long"));
@@ -293,7 +295,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     Highlighter highlighter = new Highlighter(scorer);
 
     final int docId0 = hits.scoreDocs[0].doc;
-    Document doc = searcher.doc(docId0);
+    Document doc = searcher.storedFields().document(docId0);
     String storedField = doc.get(FIELD_NAME);
 
     TokenStream stream = getAnyTokenStream(FIELD_NAME, docId0);
@@ -305,7 +307,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
         fragment);
 
     final int docId1 = hits.scoreDocs[1].doc;
-    doc = searcher.doc(docId1);
+    doc = searcher.storedFields().document(docId1);
     storedField = doc.get(FIELD_NAME);
 
     stream = getAnyTokenStream(FIELD_NAME, docId1);
@@ -373,7 +375,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     Highlighter highlighter = new Highlighter(scorer);
 
     for (int i = 0; i < hits.totalHits.value; i++) {
-      String text = searcher.doc(hits.scoreDocs[i].doc).get(FIELD_NAME);
+      String text = searcher.storedFields().document(hits.scoreDocs[i].doc).get(FIELD_NAME);
       TokenStream tokenStream = analyzer.tokenStream(FIELD_NAME, text);
       highlighter.setTextFragmenter(new SimpleFragmenter(40));
 
@@ -398,7 +400,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     assertEquals("Must have one hit", 1, hits.totalHits.value);
     for (int i = 0; i < hits.totalHits.value; i++) {
-      String text = searcher.doc(hits.scoreDocs[i].doc).get(FIELD_NAME);
+      String text = searcher.storedFields().document(hits.scoreDocs[i].doc).get(FIELD_NAME);
       TokenStream tokenStream = analyzer.tokenStream(FIELD_NAME, text);
 
       highlighter.setTextFragmenter(new SimpleSpanFragmenter(scorer, 36));
@@ -465,7 +467,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -498,7 +500,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -531,7 +533,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -560,7 +562,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -587,7 +589,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -614,7 +616,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -644,7 +646,8 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     for (int i = 0; i < hits.totalHits.value; i++) {
       String text =
           searcher
-              .doc(hits.scoreDocs[i].doc)
+              .storedFields()
+              .document(hits.scoreDocs[i].doc)
               .getField(NUMERIC_FIELD_NAME)
               .numericValue()
               .toString();
@@ -670,7 +673,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -692,7 +695,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
       QueryScorer scorer = new QueryScorer(query, FIELD_NAME);
@@ -726,7 +729,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -747,7 +750,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     highlighter = new Highlighter(this, scorer);
 
     for (int i = 0; i < hits.totalHits.value; i++) {
-      String text = searcher.doc(hits.scoreDocs[i].doc).get(FIELD_NAME);
+      String text = searcher.storedFields().document(hits.scoreDocs[i].doc).get(FIELD_NAME);
       TokenStream tokenStream = analyzer.tokenStream(FIELD_NAME, text);
 
       highlighter.setTextFragmenter(new SimpleSpanFragmenter(scorer, 20));
@@ -775,7 +778,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -856,7 +859,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     int maxNumFragmentsRequired = 2;
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -1091,7 +1094,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
         new WildcardQuery(
             new Term(FIELD_NAME, "ken*"),
             Operations.DEFAULT_DETERMINIZE_WORK_LIMIT,
-            MultiTermQuery.CONSTANT_SCORE_REWRITE);
+            MultiTermQuery.CONSTANT_SCORE_BLENDED_REWRITE);
     searcher = newSearcher(reader);
     // can't rewrite ConstantScore if you want to highlight it -
     // it rewrites to ConstantScoreQuery which cannot be highlighted
@@ -1101,7 +1104,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
       int maxNumFragmentsRequired = 2;
@@ -1129,7 +1132,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
       int maxNumFragmentsRequired = 2;
@@ -1157,7 +1160,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     for (int i = 0; i < hits.totalHits.value; i++) {
       final int docId = hits.scoreDocs[i].doc;
-      final Document doc = searcher.doc(docId);
+      final Document doc = searcher.storedFields().document(docId);
       String text = doc.get(FIELD_NAME);
       TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
       int maxNumFragmentsRequired = 2;
@@ -1355,7 +1358,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
             numHighlights = 0;
             for (int i = 0; i < hits.totalHits.value; i++) {
               final int docId = hits.scoreDocs[i].doc;
-              final Document doc = searcher.doc(docId);
+              final Document doc = searcher.storedFields().document(docId);
               String text = doc.get(FIELD_NAME);
               TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -1371,7 +1374,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
             numHighlights = 0;
             for (int i = 0; i < hits.totalHits.value; i++) {
               final int docId = hits.scoreDocs[i].doc;
-              final Document doc = searcher.doc(docId);
+              final Document doc = searcher.storedFields().document(docId);
               String text = doc.get(FIELD_NAME);
               TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
               Highlighter highlighter = getHighlighter(query, FIELD_NAME, TestHighlighter.this);
@@ -1384,7 +1387,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
             numHighlights = 0;
             for (int i = 0; i < hits.totalHits.value; i++) {
               final int docId = hits.scoreDocs[i].doc;
-              final Document doc = searcher.doc(docId);
+              final Document doc = searcher.storedFields().document(docId);
               String text = doc.get(FIELD_NAME);
               TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -1422,6 +1425,17 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
         "foo"); // field "foo"
   }
 
+  public void testFieldExistsQuery() throws IOException {
+    Query q =
+        new BooleanQuery.Builder()
+            .add(new TermQuery(new Term("field", "term")), Occur.MUST)
+            .add(new FieldExistsQuery("field"), Occur.MUST)
+            .build();
+
+    WeightedSpanTermExtractor extractor = new WeightedSpanTermExtractor();
+    extractor.getWeightedSpanTerms(q, 1, new CannedTokenStream(new Token("term", 0, 4)), "field");
+  }
+
   public void testGetBestSingleFragmentWithWeights() throws Exception {
 
     TestHighlightRunner helper =
@@ -1519,7 +1533,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
             // new Highlighter(TestHighlighter.this, new QueryTermScorer(query));
 
             for (int i = 0; i < hits.totalHits.value; i++) {
-              String text = searcher.doc(hits.scoreDocs[i].doc).get(FIELD_NAME);
+              String text = searcher.storedFields().document(hits.scoreDocs[i].doc).get(FIELD_NAME);
               TokenStream tokenStream = analyzer.tokenStream(FIELD_NAME, text);
               Highlighter highlighter = getHighlighter(query, FIELD_NAME, TestHighlighter.this);
               String result = highlighter.getBestFragment(tokenStream, text);
@@ -1544,7 +1558,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
             for (int i = 0; i < hits.totalHits.value; i++) {
               final int docId = hits.scoreDocs[i].doc;
-              final Document doc = searcher.doc(docId);
+              final Document doc = searcher.storedFields().document(docId);
               String text = doc.get(FIELD_NAME);
               TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
 
@@ -1758,7 +1772,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
             for (int i = 0; i < hits.totalHits.value; i++) {
               final int docId = hits.scoreDocs[i].doc;
-              final Document doc = searcher.doc(docId);
+              final Document doc = searcher.storedFields().document(docId);
               String text = doc.get(FIELD_NAME);
               TokenStream tokenStream = getAnyTokenStream(FIELD_NAME, docId);
               Highlighter highlighter =
@@ -2148,7 +2162,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
 
     TopDocs hits = searcher.search(query, 10);
     for (int i = 0; i < hits.totalHits.value; i++) {
-      Document doc = searcher.doc(hits.scoreDocs[i].doc);
+      Document doc = searcher.storedFields().document(hits.scoreDocs[i].doc);
       String result = h.getBestFragment(a, "t_text1", doc.get("t_text1"));
       if (VERBOSE) System.out.println("result:" + result);
       assertEquals("more <B>random</B> words for second field", result);
@@ -2209,7 +2223,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
     searcher = newSearcher(reader);
     // for any multi-term queries to work (prefix, wildcard, range,fuzzy etc)
     // you must use a rewritten query!
-    query = unReWrittenQuery.rewrite(reader);
+    query = unReWrittenQuery.rewrite(searcher);
     if (VERBOSE) System.out.println("Searching for: " + query.toString(FIELD_NAME));
     hits = searcher.search(query, 1000);
   }
@@ -2217,7 +2231,7 @@ public class TestHighlighter extends BaseTokenStreamTestCase implements Formatte
   public void assertExpectedHighlightCount(
       final int maxNumFragmentsRequired, final int expectedHighlights) throws Exception {
     for (int i = 0; i < hits.totalHits.value; i++) {
-      String text = searcher.doc(hits.scoreDocs[i].doc).get(FIELD_NAME);
+      String text = searcher.storedFields().document(hits.scoreDocs[i].doc).get(FIELD_NAME);
       TokenStream tokenStream = analyzer.tokenStream(FIELD_NAME, text);
       QueryScorer scorer = new QueryScorer(query, FIELD_NAME);
       Highlighter highlighter = new Highlighter(this, scorer);
@@ -2482,20 +2496,17 @@ final class SynonymTokenizer extends TokenStream {
         boolean expandMT)
         throws Exception {
 
+      TermVectors termVectors = searcher.getIndexReader().termVectors();
       for (int i = 0; i < hits.totalHits.value; i++) {
         final int docId = hits.scoreDocs[i].doc;
-        final Document doc = searcher.doc(docId);
+        final Document doc = searcher.storedFields().document(docId);
         String text = doc.get(TestHighlighter.FIELD_NAME);
         int maxNumFragmentsRequired = 2;
         String fragmentSeparator = "...";
         Scorer scorer = null;
         TokenStream tokenStream =
             TokenSources.getTokenStream(
-                TestHighlighter.FIELD_NAME,
-                searcher.getIndexReader().getTermVectors(docId),
-                text,
-                analyzer,
-                -1);
+                TestHighlighter.FIELD_NAME, termVectors.get(docId), text, analyzer, -1);
         if (mode == QUERY) {
           scorer = new QueryScorer(query);
         } else if (mode == QUERY_TERM) {
