@@ -31,6 +31,7 @@ import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.document.Field.Store;
 import org.apache.lucene.document.IntPoint;
+import org.apache.lucene.document.KnnFloatVectorField;
 import org.apache.lucene.document.NumericDocValuesField;
 import org.apache.lucene.document.SortedDocValuesField;
 import org.apache.lucene.document.StoredField;
@@ -47,6 +48,7 @@ import org.apache.lucene.index.NoMergePolicy;
 import org.apache.lucene.index.PostingsEnum;
 import org.apache.lucene.index.ReaderUtil;
 import org.apache.lucene.index.Term;
+import org.apache.lucene.index.VectorSimilarityFunction;
 import org.apache.lucene.search.*;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.similarities.BasicStats;
@@ -93,6 +95,20 @@ public class TestBlockJoin extends LuceneTestCase {
     return job;
   }
 
+  private Document makeVector(String vectorField, String childsParent, float[] value) {
+    Document vectorDoc = new Document();
+    vectorDoc.add(new KnnFloatVectorField(vectorField, value));
+    vectorDoc.add(newStringField("my_parent_id", childsParent, Store.YES));
+    return vectorDoc;
+  }
+
+  private Document makeParent(String parentId) {
+    Document parent = new Document();
+    parent.add(newStringField("docType", "_parent", Field.Store.NO));
+    parent.add(newStringField("parent_id", parentId, Store.YES));
+    return parent;
+  }
+
   public void testEmptyChildFilter() throws Exception {
     final Directory dir = newDirectory();
     final IndexWriterConfig config = new IndexWriterConfig(new MockAnalyzer(random()));
@@ -136,22 +152,22 @@ public class TestBlockJoin extends LuceneTestCase {
     assertEquals(
         asSet("Lisa", "Frank"),
         asSet(
-            s.doc(topDocs.scoreDocs[0].doc).get("name"),
-            s.doc(topDocs.scoreDocs[1].doc).get("name")));
+            s.storedFields().document(topDocs.scoreDocs[0].doc).get("name"),
+            s.storedFields().document(topDocs.scoreDocs[1].doc).get("name")));
 
     ParentChildrenBlockJoinQuery childrenQuery =
         new ParentChildrenBlockJoinQuery(
             parentsFilter, childQuery.build(), topDocs.scoreDocs[0].doc);
     TopDocs matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     childrenQuery =
         new ParentChildrenBlockJoinQuery(
             parentsFilter, childQuery.build(), topDocs.scoreDocs[1].doc);
     matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     r.close();
     dir.close();
@@ -160,7 +176,9 @@ public class TestBlockJoin extends LuceneTestCase {
   // You must use ToParentBlockJoinSearcher if you want to do BQ SHOULD queries:
   public void testBQShouldJoinedChild() throws Exception {
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
 
@@ -208,31 +226,81 @@ public class TestBlockJoin extends LuceneTestCase {
     assertEquals(
         asSet("Lisa", "Frank"),
         asSet(
-            s.doc(topDocs.scoreDocs[0].doc).get("name"),
-            s.doc(topDocs.scoreDocs[1].doc).get("name")));
+            s.storedFields().document(topDocs.scoreDocs[0].doc).get("name"),
+            s.storedFields().document(topDocs.scoreDocs[1].doc).get("name")));
 
     ParentChildrenBlockJoinQuery childrenQuery =
         new ParentChildrenBlockJoinQuery(
             parentsFilter, childQuery.build(), topDocs.scoreDocs[0].doc);
     TopDocs matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     childrenQuery =
         new ParentChildrenBlockJoinQuery(
             parentsFilter, childQuery.build(), topDocs.scoreDocs[1].doc);
     matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     r.close();
     dir.close();
   }
 
+  public void testSimpleKnn() throws Exception {
+
+    final Directory dir = newDirectory();
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
+
+    final List<Document> docs = new ArrayList<>();
+
+    docs.add(makeVector("vector", "parent1", new float[] {1f, 2f, 3f}));
+    docs.add(makeVector("vector", "parent1", new float[] {3f, 3f, 3f}));
+    docs.add(makeParent("parent1"));
+    w.addDocuments(docs);
+
+    docs.clear();
+    docs.add(makeVector("vector", "parent2", new float[] {0f, 0f, 1f}));
+    docs.add(makeVector("vector", "parent2", new float[] {1f, 1f, 1f}));
+    docs.add(makeParent("parent2"));
+    w.addDocuments(docs);
+
+    IndexReader r = w.getReader();
+    w.close();
+    IndexSearcher s = newSearcher(r, false);
+
+    // Create a filter that defines "parent" documents in the index
+    BitSetProducer parentsFilter =
+        new QueryBitSetProducer(new TermQuery(new Term("docType", "_parent")));
+    CheckJoinIndex.check(r, parentsFilter);
+
+    DiversifyingChildrenFloatKnnVectorQuery childKnnJoin =
+        new DiversifyingChildrenFloatKnnVectorQuery(
+            "vector", new float[] {4f, 4f, 4f}, null, 3, parentsFilter);
+
+    TopDocs topDocs = s.search(childKnnJoin, 5);
+    assertEquals(2, topDocs.totalHits.value);
+    Document childDoc = s.storedFields().document(topDocs.scoreDocs[0].doc);
+    assertEquals("parent1", childDoc.get("my_parent_id"));
+    assertEquals(
+        topDocs.scoreDocs[0].score,
+        VectorSimilarityFunction.EUCLIDEAN.compare(
+            new float[] {4f, 4f, 4f}, new float[] {3f, 3f, 3f}),
+        1e-7);
+    childDoc = s.storedFields().document(topDocs.scoreDocs[1].doc);
+    assertEquals("parent2", childDoc.get("my_parent_id"));
+    r.close();
+    dir.close();
+  }
+
   public void testSimple() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
 
@@ -277,10 +345,9 @@ public class TestBlockJoin extends LuceneTestCase {
     CheckHits.checkHitCollector(random(), fullQuery.build(), "country", s, new int[] {2});
 
     TopDocs topDocs = s.search(fullQuery.build(), 1);
-
     // assertEquals(1, results.totalHitCount);
     assertEquals(1, topDocs.totalHits.value);
-    Document parentDoc = s.doc(topDocs.scoreDocs[0].doc);
+    Document parentDoc = s.storedFields().document(topDocs.scoreDocs[0].doc);
     assertEquals("Lisa", parentDoc.get("name"));
 
     ParentChildrenBlockJoinQuery childrenQuery =
@@ -288,7 +355,7 @@ public class TestBlockJoin extends LuceneTestCase {
             parentsFilter, childQuery.build(), topDocs.scoreDocs[0].doc);
     TopDocs matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     // System.out.println("TEST: now test up");
 
@@ -301,7 +368,7 @@ public class TestBlockJoin extends LuceneTestCase {
     // System.out.println("FULL: " + fullChildQuery);
     TopDocs hits = s.search(fullChildQuery.build(), 10);
     assertEquals(1, hits.totalHits.value);
-    Document childDoc = s.doc(hits.scoreDocs[0].doc);
+    Document childDoc = s.storedFields().document(hits.scoreDocs[0].doc);
     // System.out.println("CHILD = " + childDoc + " docID=" + hits.scoreDocs[0].doc);
     assertEquals("java", childDoc.get("skill"));
     assertEquals(2007, childDoc.getField("year").numericValue());
@@ -322,7 +389,9 @@ public class TestBlockJoin extends LuceneTestCase {
   public void testSimpleFilter() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
     docs.add(makeJob("java", 2007));
@@ -399,7 +468,7 @@ public class TestBlockJoin extends LuceneTestCase {
             .build();
     TopDocs ukOnly = s.search(query, 1);
     assertEquals("has filter - single passed", 1, ukOnly.totalHits.value);
-    assertEquals("Lisa", r.document(ukOnly.scoreDocs[0].doc).get("name"));
+    assertEquals("Lisa", r.storedFields().document(ukOnly.scoreDocs[0].doc).get("name"));
 
     query =
         new BooleanQuery.Builder()
@@ -409,7 +478,7 @@ public class TestBlockJoin extends LuceneTestCase {
     // looking for US candidates
     TopDocs usThen = s.search(query, 1);
     assertEquals("has filter - single passed", 1, usThen.totalHits.value);
-    assertEquals("Frank", r.document(usThen.scoreDocs[0].doc).get("name"));
+    assertEquals("Frank", r.storedFields().document(usThen.scoreDocs[0].doc).get("name"));
 
     TermQuery us = new TermQuery(new Term("country", "United States"));
     assertEquals(
@@ -449,12 +518,14 @@ public class TestBlockJoin extends LuceneTestCase {
     final int subIndex = ReaderUtil.subIndex(childDocID, leaves);
     final LeafReaderContext leaf = leaves.get(subIndex);
     final BitSet bits = parents.getBitSet(leaf);
-    return leaf.reader().document(bits.nextSetBit(childDocID - leaf.docBase));
+    return leaf.reader().storedFields().document(bits.nextSetBit(childDocID - leaf.docBase));
   }
 
   public void testBoostBug() throws Exception {
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
     IndexReader r = w.getReader();
     w.close();
     IndexSearcher s = newSearcher(r);
@@ -547,8 +618,14 @@ public class TestBlockJoin extends LuceneTestCase {
     final List<Integer> toDelete = new ArrayList<>();
 
     // TODO: parallel star join, nested join cases too!
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
-    final RandomIndexWriter joinW = new RandomIndexWriter(random(), joinDir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
+    final RandomIndexWriter joinW =
+        new RandomIndexWriter(
+            random(),
+            joinDir,
+            newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
     for (int parentDocID = 0; parentDocID < numParentDocs; parentDocID++) {
       Document parentDoc = new Document();
       Document parentJoinDoc = new Document();
@@ -669,7 +746,7 @@ public class TestBlockJoin extends LuceneTestCase {
             "  docID="
                 + docIDX
                 + " doc="
-                + joinR.document(docIDX)
+                + joinR.storedFields().document(docIDX)
                 + " deleted?="
                 + (liveDocs != null && liveDocs.get(docIDX) == false));
       }
@@ -817,7 +894,7 @@ public class TestBlockJoin extends LuceneTestCase {
                 + parentAndChildSort);
         final ScoreDoc[] hits = results.scoreDocs;
         for (int hitIDX = 0; hitIDX < hits.length; hitIDX++) {
-          final Document doc = s.doc(hits[hitIDX].doc);
+          final Document doc = s.storedFields().document(hits[hitIDX].doc);
           // System.out.println("  score=" + hits[hitIDX].score + " parentID=" + doc.get("parentID")
           // + " childID=" + doc.get("childID") + " (docID=" + hits[hitIDX].doc + ")");
           System.out.println(
@@ -849,7 +926,7 @@ public class TestBlockJoin extends LuceneTestCase {
         ParentChildrenBlockJoinQuery childrenQuery =
             new ParentChildrenBlockJoinQuery(parentsFilter, childQuery, parentHit.doc);
         TopDocs childTopDocs = joinS.search(childrenQuery, maxNumChildrenPerParent, childSort);
-        final Document parentDoc = joinS.doc(parentHit.doc);
+        final Document parentDoc = joinS.storedFields().document(parentHit.doc);
         joinResults.put(Integer.valueOf(parentDoc.get("parentID")), childTopDocs);
       }
 
@@ -866,7 +943,7 @@ public class TestBlockJoin extends LuceneTestCase {
           System.out.println(
               "  group parentID=" + entry.getKey() + " (docID=" + entry.getKey() + ")");
           for (ScoreDoc childHit : entry.getValue().scoreDocs) {
-            final Document doc = joinS.doc(childHit.doc);
+            final Document doc = joinS.storedFields().document(childHit.doc);
             //              System.out.println("    score=" + childHit.score + " childID=" +
             // doc.get("childID") + " (docID=" + childHit.doc + ")");
             System.out.println(
@@ -888,15 +965,13 @@ public class TestBlockJoin extends LuceneTestCase {
         TopDocs b = joinS.search(childJoinQuery, 10);
         for (ScoreDoc hit : b.scoreDocs) {
           Explanation explanation = joinS.explain(childJoinQuery, hit.doc);
-          Document document = joinS.doc(hit.doc - 1);
+          Document document = joinS.storedFields().document(hit.doc - 1);
           int childId = Integer.parseInt(document.get("childID"));
-          // System.out.println("  hit docID=" + hit.doc + " childId=" + childId + " parentId=" +
-          // document.get("parentID"));
           assertTrue(explanation.isMatch());
           assertEquals(hit.score, explanation.getValue().doubleValue(), 0.0f);
           Matcher m =
               Pattern.compile(
-                      "Score based on ([0-9]+) child docs in range from ([0-9]+) to ([0-9]+), best match:")
+                      "Score based on ([0-9]+) child docs in range from ([0-9]+) to ([0-9]+), using score mode (None|Avg|Min|Max|Total)")
                   .matcher(explanation.getDescription());
           assertTrue("Block Join description not matches", m.matches());
           assertTrue("Matched children not positive", Integer.parseInt(m.group(1)) > 0);
@@ -1040,7 +1115,7 @@ public class TestBlockJoin extends LuceneTestCase {
       if (VERBOSE) {
         System.out.println("  " + results2.totalHits.value + " totalHits:");
         for (ScoreDoc sd : results2.scoreDocs) {
-          final Document doc = s.doc(sd.doc);
+          final Document doc = s.storedFields().document(sd.doc);
           System.out.println(
               "  childID="
                   + doc.get("childID")
@@ -1060,7 +1135,7 @@ public class TestBlockJoin extends LuceneTestCase {
       if (VERBOSE) {
         System.out.println("  " + joinResults2.totalHits.value + " totalHits:");
         for (ScoreDoc sd : joinResults2.scoreDocs) {
-          final Document doc = joinS.doc(sd.doc);
+          final Document doc = joinS.storedFields().document(sd.doc);
           final Document parentDoc = getParentDoc(joinR, parentsFilter, sd.doc);
           System.out.println(
               "  childID="
@@ -1088,8 +1163,8 @@ public class TestBlockJoin extends LuceneTestCase {
     for (int hitCount = 0; hitCount < results.scoreDocs.length; hitCount++) {
       ScoreDoc hit = results.scoreDocs[hitCount];
       ScoreDoc joinHit = joinResults.scoreDocs[hitCount];
-      Document doc1 = r.document(hit.doc);
-      Document doc2 = joinR.document(joinHit.doc);
+      Document doc1 = r.storedFields().document(hit.doc);
+      Document doc2 = joinR.storedFields().document(joinHit.doc);
       assertEquals("hit " + hitCount + " differs", doc1.get("childID"), doc2.get("childID"));
       // don't compare scores -- they are expected to differ
 
@@ -1109,7 +1184,7 @@ public class TestBlockJoin extends LuceneTestCase {
     int childHitSlot = 0;
     TopDocs childHits = new TopDocs(new TotalHits(0, TotalHits.Relation.EQUAL_TO), new ScoreDoc[0]);
     for (ScoreDoc controlHit : controlHits.scoreDocs) {
-      Document controlDoc = r.document(controlHit.doc);
+      Document controlDoc = r.storedFields().document(controlHit.doc);
       int parentID = Integer.parseInt(controlDoc.get("parentID"));
       if (parentID != currentParentID) {
         assertEquals(childHitSlot, childHits.scoreDocs.length);
@@ -1119,7 +1194,7 @@ public class TestBlockJoin extends LuceneTestCase {
       }
 
       String controlChildID = controlDoc.get("childID");
-      Document childDoc = joinR.document(childHits.scoreDocs[childHitSlot++].doc);
+      Document childDoc = joinR.storedFields().document(childHits.scoreDocs[childHitSlot++].doc);
       String childID = childDoc.get("childID");
       assertEquals(controlChildID, childID);
     }
@@ -1128,7 +1203,9 @@ public class TestBlockJoin extends LuceneTestCase {
   public void testMultiChildTypes() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
 
@@ -1176,7 +1253,7 @@ public class TestBlockJoin extends LuceneTestCase {
 
     final TopDocs topDocs = s.search(fullQuery.build(), 10);
     assertEquals(1, topDocs.totalHits.value);
-    Document parentDoc = s.doc(topDocs.scoreDocs[0].doc);
+    Document parentDoc = s.storedFields().document(topDocs.scoreDocs[0].doc);
     assertEquals("Lisa", parentDoc.get("name"));
 
     ParentChildrenBlockJoinQuery childrenQuery =
@@ -1184,14 +1261,15 @@ public class TestBlockJoin extends LuceneTestCase {
             parentsFilter, childJobQuery.build(), topDocs.scoreDocs[0].doc);
     TopDocs matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("java", s.doc(matchingChildren.scoreDocs[0].doc).get("skill"));
+    assertEquals("java", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("skill"));
 
     childrenQuery =
         new ParentChildrenBlockJoinQuery(
             parentsFilter, childQualificationQuery.build(), topDocs.scoreDocs[0].doc);
     matchingChildren = s.search(childrenQuery, 1);
     assertEquals(1, matchingChildren.totalHits.value);
-    assertEquals("maths", s.doc(matchingChildren.scoreDocs[0].doc).get("qualification"));
+    assertEquals(
+        "maths", s.storedFields().document(matchingChildren.scoreDocs[0].doc).get("qualification"));
 
     r.close();
     dir.close();
@@ -1199,7 +1277,9 @@ public class TestBlockJoin extends LuceneTestCase {
 
   public void testAdvanceSingleParentSingleChild() throws Exception {
     Directory dir = newDirectory();
-    RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
     Document childDoc = new Document();
     childDoc.add(newStringField("child", "1", Field.Store.NO));
     Document parentDoc = new Document();
@@ -1262,7 +1342,9 @@ public class TestBlockJoin extends LuceneTestCase {
   // LUCENE-4968
   public void testChildQueryNeverMatches() throws Exception {
     Directory d = newDirectory();
-    RandomIndexWriter w = new RandomIndexWriter(random(), d);
+    RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), d, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
     Document parent = new Document();
     parent.add(new StoredField("parentID", "0"));
     parent.add(new SortedDocValuesField("parentID", new BytesRef("0")));
@@ -1332,7 +1414,9 @@ public class TestBlockJoin extends LuceneTestCase {
   public void testAdvanceSingleDeletedParentNoChild() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     // First doc with 1 children
     Document parentDoc = new Document();
@@ -1377,7 +1461,9 @@ public class TestBlockJoin extends LuceneTestCase {
 
   public void testIntersectionWithRandomApproximation() throws IOException {
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final int numBlocks = atLeast(100);
     for (int i = 0; i < numBlocks; ++i) {
@@ -1423,7 +1509,9 @@ public class TestBlockJoin extends LuceneTestCase {
   // delete documents to simulate FilteredQuery applying a filter as acceptDocs
   public void testParentScoringBug() throws Exception {
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
     docs.add(makeJob("java", 2007));
@@ -1461,7 +1549,9 @@ public class TestBlockJoin extends LuceneTestCase {
 
   public void testToChildBlockJoinQueryExplain() throws Exception {
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     final List<Document> docs = new ArrayList<>();
     docs.add(makeJob("java", 2007));
@@ -1503,7 +1593,9 @@ public class TestBlockJoin extends LuceneTestCase {
   public void testToChildInitialAdvanceParentButNoKids() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     // degenerate case: first doc has no children
     w.addDocument(makeResume("first", "nokids"));
@@ -1541,7 +1633,9 @@ public class TestBlockJoin extends LuceneTestCase {
   public void testMultiChildQueriesOfDiffParentLevels() throws Exception {
 
     final Directory dir = newDirectory();
-    final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
+    final RandomIndexWriter w =
+        new RandomIndexWriter(
+            random(), dir, newIndexWriterConfig().setMergePolicy(newMergePolicy(random(), false)));
 
     // randomly generate resume->jobs[]->qualifications[]
     final int numResumes = atLeast(100);
@@ -1593,7 +1687,7 @@ public class TestBlockJoin extends LuceneTestCase {
 
       for (ScoreDoc sd : hits.scoreDocs) {
         // since we're looking for children of jobs, all results must be qualifications
-        String q = r.document(sd.doc).get("qualification");
+        String q = r.storedFields().document(sd.doc).get("qualification");
         assertNotNull(sd.doc + " has no qualification", q);
         assertTrue(q + " MUST contain jv" + qjv, q.contains("jv" + qjv));
         assertTrue(q + " MUST contain rv" + qrv, q.contains("rv" + qrv));
@@ -1620,7 +1714,12 @@ public class TestBlockJoin extends LuceneTestCase {
         };
     Directory dir = newDirectory();
     RandomIndexWriter w =
-        new RandomIndexWriter(random(), dir, newIndexWriterConfig().setSimilarity(sim));
+        new RandomIndexWriter(
+            random(),
+            dir,
+            newIndexWriterConfig()
+                .setSimilarity(sim)
+                .setMergePolicy(newMergePolicy(random(), false)));
     w.addDocuments(
         Arrays.asList(
             Collections.singleton(newTextField("foo", "bar bar", Store.NO)),
