@@ -41,7 +41,7 @@ public class TestPresearcherMatchCollector extends MonitorTestBase {
       PresearcherMatches<QueryMatch> matches = monitor.debug(doc, QueryMatch.SIMPLE_MATCHER);
 
       assertNotNull(matches.match("1", 0));
-      assertEquals(" field:test", matches.match("1", 0).presearcherMatches);
+      assertEquals(" field:(foo test)", matches.match("1", 0).presearcherMatches);
       assertNotNull(matches.match("1", 0).queryMatch);
 
       assertNotNull(matches.match("2", 0));
@@ -50,7 +50,7 @@ public class TestPresearcherMatchCollector extends MonitorTestBase {
       assertThat(pm, containsString("f2:(quuz)"));
 
       assertNotNull(matches.match("3", 0));
-      assertEquals(" field:foo", matches.match("3", 0).presearcherMatches);
+      assertEquals(" field:(foo test)", matches.match("3", 0).presearcherMatches);
       assertNull(matches.match("3", 0).queryMatch);
 
       assertNull(matches.match("4", 0));
