@@ -30,6 +30,7 @@ import org.apache.lucene.index.Term;
 import org.apache.lucene.index.TermState;
 import org.apache.lucene.index.TermStates;
 import org.apache.lucene.index.TermsEnum;
+import org.apache.lucene.internal.hppc.IntObjectHashMap;
 import org.apache.lucene.queries.spans.SpanNearQuery;
 import org.apache.lucene.search.BooleanClause;
 import org.apache.lucene.search.Explanation;
@@ -85,7 +86,7 @@ public class TermAutomatonQuery extends Query implements Accountable {
   private final Automaton.Builder builder;
   Automaton det;
   private final Map<BytesRef, Integer> termToID = new HashMap<>();
-  private final Map<Integer, BytesRef> idToTerm = new HashMap<>();
+  private final IntObjectHashMap<BytesRef> idToTerm = new IntObjectHashMap<>();
   private int anyTermID = -1;
 
   public TermAutomatonQuery(String field) {
@@ -208,7 +209,7 @@ public class TermAutomatonQuery extends Query implements Accountable {
   @Override
   public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
       throws IOException {
-    Map<Integer, TermStates> termStates = new HashMap<>();
+    IntObjectHashMap<TermStates> termStates = new IntObjectHashMap<>();
 
     for (Map.Entry<BytesRef, Integer> ent : termToID.entrySet()) {
       if (ent.getKey() != null) {
@@ -359,14 +360,14 @@ public class TermAutomatonQuery extends Query implements Accountable {
 
   final class TermAutomatonWeight extends Weight {
     final Automaton automaton;
-    private final Map<Integer, TermStates> termStates;
+    private final IntObjectHashMap<TermStates> termStates;
     private final Similarity.SimScorer stats;
     private final Similarity similarity;
 
     public TermAutomatonWeight(
         Automaton automaton,
         IndexSearcher searcher,
-        Map<Integer, TermStates> termStates,
+        IntObjectHashMap<TermStates> termStates,
         float boost)
         throws IOException {
       super(TermAutomatonQuery.this);
@@ -374,14 +375,13 @@ public class TermAutomatonQuery extends Query implements Accountable {
       this.termStates = termStates;
       this.similarity = searcher.getSimilarity();
       List<TermStatistics> allTermStats = new ArrayList<>();
-      for (Map.Entry<Integer, BytesRef> ent : idToTerm.entrySet()) {
-        Integer termID = ent.getKey();
-        if (ent.getValue() != null) {
-          TermStates ts = termStates.get(termID);
+      for (IntObjectHashMap.IntObjectCursor<BytesRef> ent : idToTerm) {
+        if (ent.value != null) {
+          TermStates ts = termStates.get(ent.key);
           if (ts.docFreq() > 0) {
             allTermStats.add(
                 searcher.termStatistics(
-                    new Term(field, ent.getValue()), ts.docFreq(), ts.totalTermFreq()));
+                    new Term(field, ent.value), ts.docFreq(), ts.totalTermFreq()));
           }
         }
       }
@@ -409,18 +409,18 @@ public class TermAutomatonQuery extends Query implements Accountable {
       EnumAndScorer[] enums = new EnumAndScorer[idToTerm.size()];
 
       boolean any = false;
-      for (Map.Entry<Integer, TermStates> ent : termStates.entrySet()) {
-        TermStates termStates = ent.getValue();
+      for (IntObjectHashMap.IntObjectCursor<TermStates> ent : termStates) {
+        TermStates termStates = ent.value;
         assert termStates.wasBuiltFor(ReaderUtil.getTopLevelContext(context))
             : "The top-reader used to create Weight is not the same as the current reader's top-reader ("
                 + ReaderUtil.getTopLevelContext(context);
-        BytesRef term = idToTerm.get(ent.getKey());
+        BytesRef term = idToTerm.get(ent.key);
         TermState state = termStates.get(context);
         if (state != null) {
           TermsEnum termsEnum = context.reader().terms(field).iterator();
           termsEnum.seekExact(term, state);
-          enums[ent.getKey()] =
-              new EnumAndScorer(ent.getKey(), termsEnum.postings(null, PostingsEnum.POSITIONS));
+          enums[ent.key] =
+              new EnumAndScorer(ent.key, termsEnum.postings(null, PostingsEnum.POSITIONS));
           any = true;
         }
       }
