@@ -147,10 +147,13 @@ public final class NRTSuggester implements Accountable {
 
     final CharsRefBuilder spare = new CharsRefBuilder();
 
-    Comparator<Pair<Long, BytesRef>> comparator = getComparator();
     Util.TopNSearcher<Pair<Long, BytesRef>> searcher =
-        new Util.TopNSearcher<Pair<Long, BytesRef>>(
-            fst, topN, queueSize, comparator, new ScoringPathComparator(scorer)) {
+        new Util.TopNSearcher<>(
+            fst,
+            topN,
+            queueSize,
+            (o1, o2) -> Long.compare(o1.output1, o2.output1),
+            new ScoringPathComparator(scorer)) {
 
           private final ByteArrayDataInput scratchInput = new ByteArrayDataInput();
 
@@ -281,15 +284,6 @@ public final class NRTSuggester implements Accountable {
     }
   }
 
-  private static Comparator<Pair<Long, BytesRef>> getComparator() {
-    return new Comparator<Pair<Long, BytesRef>>() {
-      @Override
-      public int compare(Pair<Long, BytesRef> o1, Pair<Long, BytesRef> o2) {
-        return Long.compare(o1.output1, o2.output1);
-      }
-    };
-  }
-
   /**
    * Simple heuristics to try to avoid over-pruning potential suggestions by the TopNSearcher. Since
    * suggestion entries can be rejected if they belong to a deleted document, the length of the
@@ -325,7 +319,10 @@ public final class NRTSuggester implements Accountable {
       case OFF_HEAP:
         return true;
       case AUTO:
-        return input instanceof ByteBufferIndexInput;
+        // TODO: Make this less hacky to maybe expose "off-heap" feature using a marker interface on
+        // the IndexInput
+        return input instanceof ByteBufferIndexInput
+            || input.getClass().getName().contains(".MemorySegmentIndexInput");
       default:
         throw new IllegalStateException("unknown enum constant: " + fstLoadMode);
     }
@@ -340,11 +337,10 @@ public final class NRTSuggester implements Accountable {
     PairOutputs<Long, BytesRef> outputs =
         new PairOutputs<>(PositiveIntOutputs.getSingleton(), ByteSequenceOutputs.getSingleton());
     if (shouldLoadFSTOffHeap(input, fstLoadMode)) {
-      OffHeapFSTStore store = new OffHeapFSTStore();
-      IndexInput clone = input.clone();
-      clone.seek(input.getFilePointer());
-      fst = new FST<>(FST.readMetadata(clone, outputs), clone, store);
-      input.seek(clone.getFilePointer() + store.size());
+      final FST.FSTMetadata<Pair<Long, BytesRef>> fstMetadata = FST.readMetadata(input, outputs);
+      OffHeapFSTStore store = new OffHeapFSTStore(input, input.getFilePointer(), fstMetadata);
+      fst = FST.fromFSTReader(fstMetadata, store);
+      input.skipBytes(store.size());
     } else {
       fst = new FST<>(FST.readMetadata(input, outputs), input);
     }
