@@ -39,7 +39,10 @@ import org.apache.lucene.index.PostingsEnum;
 import org.apache.lucene.index.SortedDocValues;
 import org.apache.lucene.index.SortedNumericDocValues;
 import org.apache.lucene.index.SortedSetDocValues;
+import org.apache.lucene.index.StoredFieldVisitor;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.TermState;
+import org.apache.lucene.index.TermVectors;
 import org.apache.lucene.index.Terms;
 import org.apache.lucene.index.TermsEnum;
 import org.apache.lucene.internal.tests.IndexPackageAccess;
@@ -113,14 +116,60 @@ public class AssertingLeafReader extends FilterLeafReader {
     return fields == null ? null : new AssertingFields(fields);
   }
 
+  @Override
+  public TermVectors termVectors() throws IOException {
+    return new AssertingTermVectors(super.termVectors());
+  }
+
+  @Override
+  public StoredFields storedFields() throws IOException {
+    return new AssertingStoredFields(super.storedFields());
+  }
+
+  /** Wraps a StoredFields but with additional asserts */
+  public static class AssertingStoredFields extends StoredFields {
+    private final StoredFields in;
+    private final Thread creationThread = Thread.currentThread();
+
+    public AssertingStoredFields(StoredFields in) {
+      this.in = in;
+    }
+
+    @Override
+    public void document(int docID, StoredFieldVisitor visitor) throws IOException {
+      assertThread("StoredFields", creationThread);
+      in.document(docID, visitor);
+    }
+  }
+
+  /** Wraps a TermVectors but with additional asserts */
+  public static class AssertingTermVectors extends TermVectors {
+    private final TermVectors in;
+    private final Thread creationThread = Thread.currentThread();
+
+    public AssertingTermVectors(TermVectors in) {
+      this.in = in;
+    }
+
+    @Override
+    public Fields get(int doc) throws IOException {
+      assertThread("TermVectors", creationThread);
+      Fields fields = in.get(doc);
+      return fields == null ? null : new AssertingFields(fields);
+    }
+  }
+
   /** Wraps a Fields but with additional asserts */
   public static class AssertingFields extends FilterFields {
+    private final Thread creationThread = Thread.currentThread();
+
     public AssertingFields(Fields in) {
       super(in);
     }
 
     @Override
     public Iterator<String> iterator() {
+      assertThread("Fields", creationThread);
       Iterator<String> iterator = super.iterator();
       assert iterator != null;
       return iterator;
@@ -128,6 +177,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public Terms terms(String field) throws IOException {
+      assertThread("Fields", creationThread);
       Terms terms = super.terms(field);
       return terms == null ? null : new AssertingTerms(terms);
     }
@@ -135,12 +185,15 @@ public class AssertingLeafReader extends FilterLeafReader {
 
   /** Wraps a Terms but with additional asserts */
   public static class AssertingTerms extends FilterTerms {
+    private final Thread creationThread = Thread.currentThread();
+
     public AssertingTerms(Terms in) {
       super(in);
     }
 
     @Override
     public TermsEnum intersect(CompiledAutomaton automaton, BytesRef bytes) throws IOException {
+      assertThread("Terms", creationThread);
       TermsEnum termsEnum = in.intersect(automaton, bytes);
       assert termsEnum != null;
       assert bytes == null || bytes.isValid();
@@ -149,6 +202,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public BytesRef getMin() throws IOException {
+      assertThread("Terms", creationThread);
       BytesRef v = in.getMin();
       assert v == null || v.isValid();
       return v;
@@ -156,6 +210,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public BytesRef getMax() throws IOException {
+      assertThread("Terms", creationThread);
       BytesRef v = in.getMax();
       assert v == null || v.isValid();
       return v;
@@ -163,6 +218,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public int getDocCount() throws IOException {
+      assertThread("Terms", creationThread);
       final int docCount = in.getDocCount();
       assert docCount > 0;
       return docCount;
@@ -170,6 +226,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public long getSumDocFreq() throws IOException {
+      assertThread("Terms", creationThread);
       final long sumDf = in.getSumDocFreq();
       assert sumDf >= getDocCount();
       return sumDf;
@@ -177,6 +234,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public long getSumTotalTermFreq() throws IOException {
+      assertThread("Terms", creationThread);
       final long sumTtf = in.getSumTotalTermFreq();
       if (hasFreqs() == false) {
         assert sumTtf == in.getSumDocFreq();
@@ -187,6 +245,7 @@ public class AssertingLeafReader extends FilterLeafReader {
 
     @Override
     public TermsEnum iterator() throws IOException {
+      assertThread("Terms", creationThread);
       TermsEnum termsEnum = super.iterator();
       assert termsEnum != null;
       return new AssertingTermsEnum(termsEnum, hasFreqs());
