@@ -53,6 +53,7 @@ import org.apache.lucene.tests.codecs.bloom.TestBloomFilteredLucenePostings;
 import org.apache.lucene.tests.codecs.mockrandom.MockRandomPostingsFormat;
 import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.tests.util.TestUtil;
+import org.apache.lucene.util.IORunnable;
 import org.apache.lucene.util.bkd.BKDConfig;
 import org.apache.lucene.util.bkd.BKDWriter;
 
@@ -149,7 +150,7 @@ public class RandomCodec extends AssertingCodec {
 
                   // We could have 0 points on merge since all docs with dimensional fields may be
                   // deleted:
-                  Runnable finalizer = writer.finish(metaOut, indexOut, dataOut);
+                  IORunnable finalizer = writer.finish(metaOut, indexOut, dataOut);
                   if (finalizer != null) {
                     metaOut.writeInt(fieldInfo.number);
                     finalizer.run();
