@@ -39,6 +39,7 @@ import org.apache.lucene.index.IndexableField;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.MultiTerms;
 import org.apache.lucene.index.SegmentReader;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.index.Terms;
 import org.apache.lucene.index.TermsEnum;
@@ -103,7 +104,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
   protected void releaseSearcher(IndexSearcher s) throws Exception {}
 
   // Called once to run searching
-  protected abstract void doSearching(ExecutorService es, long stopTime) throws Exception;
+  protected abstract void doSearching(ExecutorService es, int maxIterations) throws Exception;
 
   protected Directory getDirectory(Directory in) {
     return in;
@@ -137,7 +138,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
   private Thread[] launchIndexingThreads(
       final LineFileDocs docs,
       int numThreads,
-      final long stopTime,
+      final int maxIterations,
       final Set<String> delIDs,
       final Set<String> delPackIDs,
       final List<SubDocs> allSubDocs) {
@@ -151,7 +152,8 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
               // deleting anothers added docs works:
               final List<String> toDeleteIDs = new ArrayList<>();
               final List<SubDocs> toDeleteSubDocs = new ArrayList<>();
-              while (System.currentTimeMillis() < stopTime && !failed.get()) {
+              int iterations = 0;
+              while (++iterations < maxIterations && !failed.get()) {
                 try {
 
                   // Occasional longish pause if running
@@ -372,7 +374,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
     return threads;
   }
 
-  protected void runSearchThreads(final long stopTimeMS) throws Exception {
+  protected void runSearchThreads(final int maxIterations) throws Exception {
     final int numThreads = TEST_NIGHTLY ? TestUtil.nextInt(random(), 1, 5) : 2;
     final Thread[] searchThreads = new Thread[numThreads];
     final AtomicLong totHits = new AtomicLong();
@@ -389,7 +391,8 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
               if (VERBOSE) {
                 System.out.println(Thread.currentThread().getName() + ": launch search thread");
               }
-              while (System.currentTimeMillis() < stopTimeMS && !failed.get()) {
+              int iterations = 0;
+              while (++iterations < maxIterations && !failed.get()) {
                 try {
                   final IndexSearcher s = getCurrentSearcher();
                   try {
@@ -434,7 +437,8 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
                         trigger = totTermCount.get() / 30;
                         shift = random().nextInt(trigger);
                       }
-                      while (System.currentTimeMillis() < stopTimeMS) {
+                      int iters = 0;
+                      while (++iters < maxIterations) {
                         BytesRef term = termsEnum.next();
                         if (term == null) {
                           totTermCount.set(seenTermCount);
@@ -495,7 +499,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
     delCount.set(0);
     packCount.set(0);
 
-    final long t0 = System.currentTimeMillis();
+    final long t0 = System.nanoTime();
 
     Random random = new Random(random().nextLong());
     final LineFileDocs docs = new LineFileDocs(random);
@@ -526,9 +530,10 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
           final Bits liveDocs = reader.getLiveDocs();
           int sum = 0;
           final int inc = Math.max(1, maxDoc / 50);
+          StoredFields storedFields = reader.storedFields();
           for (int docID = 0; docID < maxDoc; docID += inc) {
             if (liveDocs == null || liveDocs.get(docID)) {
-              final Document doc = reader.document(docID);
+              final Document doc = storedFields.document(docID);
               sum += doc.getFields().size();
             }
           }
@@ -565,33 +570,35 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
 
     final int NUM_INDEX_THREADS = TestUtil.nextInt(random(), 2, 4);
 
-    final int RUN_TIME_MSEC = LuceneTestCase.TEST_NIGHTLY ? 300000 : 100 * RANDOM_MULTIPLIER;
+    final int MAX_ITERATIONS = LuceneTestCase.TEST_NIGHTLY ? 200 : 10 * RANDOM_MULTIPLIER;
 
     final Set<String> delIDs = Collections.synchronizedSet(new HashSet<String>());
     final Set<String> delPackIDs = Collections.synchronizedSet(new HashSet<String>());
     final List<SubDocs> allSubDocs = Collections.synchronizedList(new ArrayList<SubDocs>());
 
-    final long stopTime = System.currentTimeMillis() + RUN_TIME_MSEC;
-
     final Thread[] indexThreads =
-        launchIndexingThreads(docs, NUM_INDEX_THREADS, stopTime, delIDs, delPackIDs, allSubDocs);
+        launchIndexingThreads(
+            docs, NUM_INDEX_THREADS, MAX_ITERATIONS, delIDs, delPackIDs, allSubDocs);
 
     if (VERBOSE) {
       System.out.println(
           "TEST: DONE start "
               + NUM_INDEX_THREADS
               + " indexing threads ["
-              + (System.currentTimeMillis() - t0)
+              + TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - t0)
               + " ms]");
     }
 
     // Let index build up a bit
     Thread.sleep(100);
 
-    doSearching(es, stopTime);
+    doSearching(es, MAX_ITERATIONS);
 
     if (VERBOSE) {
-      System.out.println("TEST: all searching done [" + (System.currentTimeMillis() - t0) + " ms]");
+      System.out.println(
+          "TEST: all searching done ["
+              + TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - t0)
+              + " ms]");
     }
 
     for (Thread thread : indexThreads) {
@@ -601,7 +608,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
     if (VERBOSE) {
       System.out.println(
           "TEST: done join indexing threads ["
-              + (System.currentTimeMillis() - t0)
+              + TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - t0)
               + " ms]; addCount="
               + addCount
               + " delCount="
@@ -649,6 +656,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
     // Verify: make sure each group of sub-docs are still in docID order:
     for (SubDocs subDocs : allSubDocs) {
       TopDocs hits = s.search(new TermQuery(new Term("packID", subDocs.packID)), 20);
+      StoredFields storedFields = s.storedFields();
       if (!subDocs.deleted) {
         // We sort by relevance but the scores should be identical so sort falls back to by docID:
         if (hits.totalHits.value != subDocs.subIDs.size()) {
@@ -671,7 +679,7 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
               startDocID = docID;
             }
             lastDocID = docID;
-            final Document doc = s.doc(docID);
+            final Document doc = storedFields.document(docID);
             assertEquals(subDocs.packID, doc.get("packID"));
           }
 
@@ -762,7 +770,8 @@ public abstract class ThreadedIndexingAndSearchingTestCase extends LuceneTestCas
     dir.close();
 
     if (VERBOSE) {
-      System.out.println("TEST: done [" + (System.currentTimeMillis() - t0) + " ms]");
+      System.out.println(
+          "TEST: done [" + TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - t0) + " ms]");
     }
   }
 
