@@ -812,8 +812,17 @@ public class MockDirectoryWrapper extends BaseDirectoryWrapper {
           false);
     }
 
-    IndexInput delegateInput =
-        in.openInput(name, LuceneTestCase.newIOContext(randomState, context));
+    context = LuceneTestCase.newIOContext(randomState, context);
+    final boolean confined = context.readOnce;
+    if (name.startsWith(IndexFileNames.SEGMENTS) && confined == false) {
+      throw new RuntimeException(
+          "MockDirectoryWrapper: opening segments file ["
+              + name
+              + "] with a non-READONCE context["
+              + context
+              + "]");
+    }
+    IndexInput delegateInput = in.openInput(name, context);
 
     final IndexInput ii;
     int randomInt = randomState.nextInt(500);
@@ -822,15 +831,15 @@ public class MockDirectoryWrapper extends BaseDirectoryWrapper {
         System.out.println(
             "MockDirectoryWrapper: using SlowClosingMockIndexInputWrapper for file " + name);
       }
-      ii = new SlowClosingMockIndexInputWrapper(this, name, delegateInput);
+      ii = new SlowClosingMockIndexInputWrapper(this, name, delegateInput, confined);
     } else if (useSlowOpenClosers && randomInt == 1) {
       if (LuceneTestCase.VERBOSE) {
         System.out.println(
             "MockDirectoryWrapper: using SlowOpeningMockIndexInputWrapper for file " + name);
       }
-      ii = new SlowOpeningMockIndexInputWrapper(this, name, delegateInput);
+      ii = new SlowOpeningMockIndexInputWrapper(this, name, delegateInput, confined);
     } else {
-      ii = new MockIndexInputWrapper(this, name, delegateInput, null);
+      ii = new MockIndexInputWrapper(this, name, delegateInput, null, confined);
     }
     addFileHandle(ii, name, Handle.Input);
     return ii;
